(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/forms'), require('rxjs'), require('@thebespokepixel/es-tinycolor'), require('@angular/cdk/coercion'), require('rxjs/operators'), require('@angular/common'), require('@angular/cdk/overlay'), require('@angular/cdk/portal'), require('@angular/material/button'), require('@angular/material/form-field'), require('@angular/material/input'), require('@angular/material/icon')) :
    typeof define === 'function' && define.amd ? define('material-community-components/color-picker', ['exports', '@angular/core', '@angular/forms', 'rxjs', '@thebespokepixel/es-tinycolor', '@angular/cdk/coercion', 'rxjs/operators', '@angular/common', '@angular/cdk/overlay', '@angular/cdk/portal', '@angular/material/button', '@angular/material/form-field', '@angular/material/input', '@angular/material/icon'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global['material-community-components'] = global['material-community-components'] || {}, global['material-community-components']['color-picker'] = {}), global.ng.core, global.ng.forms, global.rxjs, global.esTinycolor, global.ng.cdk.coercion, global.rxjs.operators, global.ng.common, global.ng.cdk.overlay, global.ng.cdk.portal, global.ng.material.button, global.ng.material.formField, global.ng.material.input, global.ng.material.icon));
}(this, (function (exports, core, forms, rxjs, esTinycolor, coercion, operators, common, overlay, portal, button, formField, input, icon) { 'use strict';

    /** Contant used as empty color */
    var EMPTY_COLOR = new core.InjectionToken('empty-color');
    /** Constante to set usedColorStart from the module import */
    var USED_COLORS = new core.InjectionToken('used-colors');
    /** Customize selected color icon */
    var SELECTED_COLOR_ICON = new core.InjectionToken('selected-color-icon');
    /** Customize selected color svg icon */
    var SELECTED_COLOR_SVG_ICON = new core.InjectionToken('selected-color-svg-icon');
    /** Disable selected color icon */
    var DISABLE_SELECTED_COLOR_ICON = new core.InjectionToken('disable-selected-color-icon');
    /** Enable alpha selector **/
    var ENABLE_ALPHA_SELECTOR = new core.InjectionToken('enable-alpha-selector');
    /** Format used for color strings **/
    var COLOR_STRING_FORMAT = new core.InjectionToken('color-string-format');

    /**
     * parses a string-representation of a color with tinycolor - returns "null" when not a valid string
     */
    function parseColorString(colorString) {
        var color = new esTinycolor.TinyColor(colorString);
        if (color.isValid()) {
            return color;
        }
        else {
            return null;
        }
    }
    /**
     * converts a TinyColor instance to format "#FFFFFF' or "#FFFFFFFF" with alpha < 1
     */
    function toHex(color) {
        if (!color) {
            return null;
        }
        if (color.getAlpha() < 1) {
            return color.toHex8String().toUpperCase();
        }
        else {
            return color.toHexString().toUpperCase();
        }
    }
    /**
     * converts a TinyColor instance to format "rgb(255, 255, 255)" when color has no alpha value and "rgba(255, 255, 255, 0.5)" when alpha < 1
     */
    function toRgb(color) {
        if (!color) {
            return null;
        }
        return color.toRgbString();
    }
    /**
     * converts a TinyColor instance to format "hsl(360, 100%, 100%)" when color has no alpha value and "hsla(360, 100%, 100%, 0.5)" when alpha < 1
     */
    function toHsl(color) {
        if (!color) {
            return null;
        }
        return color.toHslString();
    }
    /**
     * converts a TinyColor instance to certain format
     */
    function formatColor(color, format) {
        switch (format) {
            case 'hex':
                return toHex(color);
            case 'rgb':
                return toRgb(color);
            case 'hsl':
                return toHsl(color);
        }
    }

    /**
     * Directive applied to an element to make it usable as an origin for an ColorPicker.
     */
    var MccColorPickerOriginDirective = /** @class */ (function () {
        function MccColorPickerOriginDirective(elementRef, renderer, emptyColor) {
            this.elementRef = elementRef;
            this.renderer = renderer;
            this.emptyColor = emptyColor;
            /**
             * Emit focus event
             */
            this.hasFocus = new core.EventEmitter();
            /**
             * Emit changes from the origin
             */
            this.change = new rxjs.BehaviorSubject(this.elementRef.nativeElement.value);
            /**
             * Controls if focus of the input opens the color picker dialog
             */
            this.openMode = 'default';
        }
        MccColorPickerOriginDirective.prototype.onFocus = function () {
            if (this.openMode === 'openOnFocus') {
                this.hasFocus.emit();
            }
        };
        MccColorPickerOriginDirective.prototype.onKeyup = function () {
            var value = this.elementRef.nativeElement.value;
            if (!value) {
                this.writeValueFromKeyup(this.emptyColor);
            }
            else {
                var color = parseColorString(value);
                if (color) {
                    this.writeValueFromKeyup(value);
                }
            }
        };
        /**
         * This method will be called by the forms API to write to the view when
         * programmatic (model -> view) changes are requested.
         */
        MccColorPickerOriginDirective.prototype.writeValue = function (color) {
            this.renderer.setProperty(this.elementRef.nativeElement, 'value', color);
            this.change.next(color);
            if (this.propagateChanges) {
                this.propagateChanges(color);
            }
        };
        /**
         * This method will be called by the color picker
         */
        MccColorPickerOriginDirective.prototype.writeValueFromColorPicker = function (color) {
            this.renderer.setProperty(this.elementRef.nativeElement, 'value', color);
            this.propagateChanges(color);
        };
        /**
         * This method will be called from origin whe key is up
         */
        MccColorPickerOriginDirective.prototype.writeValueFromKeyup = function (color) {
            this.change.next(color);
            this.propagateChanges(color);
        };
        /**
         * This is called by the forms API on initialization so it can update the
         * form model when values propagate from the view (view -> model).
         */
        MccColorPickerOriginDirective.prototype.registerOnChange = function (fn) {
            this.propagateChanges = fn;
        };
        /**
         * This is called by the forms API on initialization so it can update the form model on blur
         */
        MccColorPickerOriginDirective.prototype.registerOnTouched = function (fn) { };
        /**
         * called by the forms API when the control status changes to or from "DISABLED"
         */
        MccColorPickerOriginDirective.prototype.setDisabledState = function (isDisabled) {
            this.renderer.setProperty(this.elementRef.nativeElement, 'disabled', isDisabled);
        };
        return MccColorPickerOriginDirective;
    }());
    MccColorPickerOriginDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[mcc-color-picker-origin], [mccColorPickerOrigin]',
                    exportAs: 'mccColorPickerOrigin',
                    providers: [
                        {
                            provide: forms.NG_VALUE_ACCESSOR,
                            useExisting: core.forwardRef(function () { return MccColorPickerOriginDirective; }),
                            multi: true
                        }
                    ]
                },] }
    ];
    MccColorPickerOriginDirective.ctorParameters = function () { return [
        { type: core.ElementRef },
        { type: core.Renderer2 },
        { type: String, decorators: [{ type: core.Inject, args: [EMPTY_COLOR,] }] }
    ]; };
    MccColorPickerOriginDirective.propDecorators = {
        openMode: [{ type: core.Input, args: ['mccColorPickerOrigin',] }],
        onFocus: [{ type: core.HostListener, args: ['focus',] }],
        onKeyup: [{ type: core.HostListener, args: ['keyup',] }]
    };

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (Object.prototype.hasOwnProperty.call(b, p))
                    d[p] = b[p]; };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            } }
            function rejected(value) { try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    var __createBinding = Object.create ? (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        Object.defineProperty(o, k2, { enumerable: true, get: function () { return m[k]; } });
    }) : (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    });
    function __exportStar(m, o) {
        for (var p in m)
            if (p !== "default" && !Object.prototype.hasOwnProperty.call(o, p))
                __createBinding(o, m, p);
    }
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    ;
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) { if (g[n])
            i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
        function resume(n, v) { try {
            step(g[n](v));
        }
        catch (e) {
            settle(q[0][3], e);
        } }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) { if (f(v), q.shift(), q.length)
            resume(q[0][0], q[0][1]); }
    }
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    ;
    var __setModuleDefault = Object.create ? (function (o, v) {
        Object.defineProperty(o, "default", { enumerable: true, value: v });
    }) : function (o, v) {
        o["default"] = v;
    };
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k))
                    __createBinding(result, mod, k);
        __setModuleDefault(result, mod);
        return result;
    }
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, privateMap) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to get private field on non-instance");
        }
        return privateMap.get(receiver);
    }
    function __classPrivateFieldSet(receiver, privateMap, value) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to set private field on non-instance");
        }
        privateMap.set(receiver, value);
        return value;
    }

    var MccColorPickerCollectionService = /** @class */ (function () {
        function MccColorPickerCollectionService(emptyColor) {
            this.emptyColor = emptyColor;
            /**
             * Hold current selected color
             */
            this._selectedColor = new rxjs.BehaviorSubject('');
        }
        /**
         * Change internal selected color
         */
        MccColorPickerCollectionService.prototype.changeSelectedColor = function (colorString) {
            if (colorString === this.emptyColor) {
                this._selectedColor.next(colorString);
            }
            var color = parseColorString(colorString);
            if (!colorString || !color) {
                return;
            }
            var clrString = formatColor(color, this.format);
            this._selectedColor.next(clrString);
        };
        /**
         * Return internal selected color
         */
        MccColorPickerCollectionService.prototype.getSelectedColor = function () {
            return this._selectedColor.asObservable();
        };
        return MccColorPickerCollectionService;
    }());
    MccColorPickerCollectionService.decorators = [
        { type: core.Injectable }
    ];
    MccColorPickerCollectionService.ctorParameters = function () { return [
        { type: String, decorators: [{ type: core.Inject, args: [EMPTY_COLOR,] }] }
    ]; };

    var MccColorPickerCollectionComponent = /** @class */ (function () {
        function MccColorPickerCollectionComponent(changeDetectorRef, colorPickerCollectionService, emptyColor) {
            this.changeDetectorRef = changeDetectorRef;
            this.colorPickerCollectionService = colorPickerCollectionService;
            this.emptyColor = emptyColor;
            this._hideEmpty = false;
            /**
             * Size limit of the collection
             */
            this.size = 30;
            this._transparent = false;
            /**
             * Emit selected color value
             */
            this.changeColor = new core.EventEmitter();
        }
        Object.defineProperty(MccColorPickerCollectionComponent.prototype, "hideEmpty", {
            /**
             * Hide empty slots
             * Empty slots are the difference between the collection size and limit
             */
            set: function (value) {
                this._hideEmpty = coercion.coerceBooleanProperty(value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerCollectionComponent.prototype, "label", {
            /**
             * Name of the collection
             */
            get: function () {
                return this._label;
            },
            set: function (value) {
                this._label = value;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerCollectionComponent.prototype, "colors", {
            /**
             * Array of colors to be displayed
             */
            get: function () {
                return this._colors;
            },
            set: function (values) {
                var _this = this;
                // map color string from user to string based on configured format
                var colors = [];
                values.forEach(function (value) {
                    if (typeof value === 'string') {
                        var clr = parseColorString(value);
                        if (clr) {
                            colors.push(formatColor(clr, _this.colorPickerCollectionService.format));
                        }
                    }
                    else {
                        var clr = value ? parseColorString(value.value) : null;
                        if (clr) {
                            colors.push({ value: formatColor(clr, _this.colorPickerCollectionService.format), text: value.text });
                        }
                    }
                });
                this._colors = colors;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerCollectionComponent.prototype, "transparent", {
            get: function () {
                return this._transparent;
            },
            /**
             * Show transparent option
             */
            set: function (value) {
                this._transparent = coercion.coerceBooleanProperty(value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerCollectionComponent.prototype, "selectedColor", {
            /**
             * Current selected color
             */
            get: function () {
                return this._selectedColor;
            },
            enumerable: false,
            configurable: true
        });
        MccColorPickerCollectionComponent.prototype.ngOnInit = function () {
            var _this = this;
            // get current selected color
            this.colorPickerCollectionService.getSelectedColor().subscribe(function (color) {
                _this._selectedColor = color;
                _this.changeDetectorRef.detectChanges();
            });
        };
        MccColorPickerCollectionComponent.prototype.ngAfterContentChecked = function () {
            if (this._colors && this._colors.length !== this.size) {
                this._colors = this._colors.slice(0, this.size).concat(new Array(this._getCollectionDiffSize()));
                this.changeDetectorRef.markForCheck();
            }
        };
        /**
         * Return the difference between the limit and the collection size.
         * Always return 0 when hideEmpty is true
         */
        MccColorPickerCollectionComponent.prototype._getCollectionDiffSize = function () {
            if (this._colors.length > this.size || this._hideEmpty) {
                return 0;
            }
            return this.size - this._colors.length;
        };
        /**
         * Remove color
         */
        MccColorPickerCollectionComponent.prototype.setTransparent = function () {
            this.changeColor.emit(this.emptyColor);
        };
        /**
         * Emit selected color value
         */
        MccColorPickerCollectionComponent.prototype.setColor = function (option) {
            var color = typeof option === 'string' ? option : option.value;
            this.colorPickerCollectionService.changeSelectedColor(color);
            this.changeColor.emit(color);
        };
        return MccColorPickerCollectionComponent;
    }());
    MccColorPickerCollectionComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'mcc-color-picker-collection',
                    template: "<div class=\"mcc-color-picker-collection\" role=\"listbox\" aria-label=\"Select a color\">\n  <h3>{{ label }}</h3>\n\n  <button\n    mat-mini-fab\n    *ngIf=\"transparent\"\n    class=\"mcc-color-picker-remove-color mat-elevation-z0\"\n    role=\"option\"\n    aria-label=\"transparent\"\n    (click)=\"setTransparent()\"\n  >\n    <span class=\"mcc-color-picker-transparent\"></span>\n  </button>\n\n  <mcc-color-picker-option\n    *ngFor=\"let color of colors\"\n    [color]=\"color\"\n    [selectedColor]=\"selectedColor\"\n    (colorSelected)=\"setColor($event)\"\n  >\n  </mcc-color-picker-option>\n</div>\n",
                    preserveWhitespaces: false,
                    changeDetection: core.ChangeDetectionStrategy.OnPush,
                    styles: ["button{cursor:pointer;height:22px;margin:3px;width:22px}.mcc-color-picker-collection{min-height:50px;padding:10px 18px}.mcc-color-picker-collection h3{color:rgba(0,0,0,.87);font-size:12px;font-weight:700;margin:0 0 10px;text-transform:uppercase}.mcc-color-picker-remove-color{background-color:#fff;border:1px solid #e1e1e1}.mcc-color-picker-remove-color .mcc-color-picker-transparent{-webkit-transform:translateY(-4px) translateX(0) rotate(45deg);border-bottom:1px solid red;display:block;height:1px;transform:translateY(-4px) translateX(0) rotate(45deg);width:20px}"]
                },] }
    ];
    MccColorPickerCollectionComponent.ctorParameters = function () { return [
        { type: core.ChangeDetectorRef },
        { type: MccColorPickerCollectionService },
        { type: String, decorators: [{ type: core.Inject, args: [EMPTY_COLOR,] }] }
    ]; };
    MccColorPickerCollectionComponent.propDecorators = {
        hideEmpty: [{ type: core.Input }],
        label: [{ type: core.Input }],
        colors: [{ type: core.Input }],
        size: [{ type: core.Input }],
        transparent: [{ type: core.Input }],
        changeColor: [{ type: core.Output }]
    };

    var MccColorPickerService = /** @class */ (function () {
        function MccColorPickerService(emptyColor, usedColors) {
            this.emptyColor = emptyColor;
            this.usedColors = usedColors;
            /**
             * Array of all used colors
             */
            this._usedColors = new rxjs.BehaviorSubject([]);
            // map color string from user to string based on configured format
            var colors = [];
            usedColors.forEach(function (value) {
                var clr = parseColorString(value);
                if (clr) {
                    colors.push(value);
                }
            });
            this._usedColors.next(colors);
        }
        /**
         * Add new color to used colors
         */
        MccColorPickerService.prototype.addColor = function (colorString) {
            var color = parseColorString(colorString);
            if (!colorString || !color) {
                return;
            }
            var usedColors = this._usedColors.getValue();
            if (!usedColors.includes(colorString)) {
                usedColors.push(colorString);
                this._usedColors.next(usedColors);
            }
        };
        /**
         * Return Observable of colors
         */
        MccColorPickerService.prototype.getUsedColors = function () {
            return this._usedColors.asObservable();
        };
        /**
         * Reset the array of used colors
         */
        MccColorPickerService.prototype.resetUsedColors = function () {
            this._usedColors.next([]);
        };
        return MccColorPickerService;
    }());
    MccColorPickerService.decorators = [
        { type: core.Injectable }
    ];
    MccColorPickerService.ctorParameters = function () { return [
        { type: String, decorators: [{ type: core.Inject, args: [EMPTY_COLOR,] }] },
        { type: Array, decorators: [{ type: core.Inject, args: [USED_COLORS,] }] }
    ]; };

    var MccColorPickerComponent = /** @class */ (function () {
        function MccColorPickerComponent(elementRef, changeDetectorRef, colorPickerService, colorPickerCollectionService, emptyColor, showAlphaSelector, colorStringFormat) {
            this.elementRef = elementRef;
            this.changeDetectorRef = changeDetectorRef;
            this.colorPickerService = colorPickerService;
            this.colorPickerCollectionService = colorPickerCollectionService;
            this.emptyColor = emptyColor;
            this._disabled = false;
            this._usedColorLabel = 'Used Colors';
            this._reverseUsedColor = false;
            this._usedColorsPosition = 'top';
            this._hideHexForms = false;
            this._hideEmpty = false;
            this._hideTransparent = false;
            this._hideUsedColors = false;
            this._isOpen = false;
            this._overlay = true;
            this._hideButtons = false;
            this._colorPickerSelectorHeight = 170;
            this._hideColorPickerSelector = false;
            /**
             * Set the size of the used colors
             */
            this.usedSizeColors = 30;
            /**
             * Change btnCancel label
             */
            this.btnCancelLabel = 'Cancel';
            /**
             * Change btnConfirm label
             */
            this.btnConfirmLabel = 'Confirm';
            this._alpha = false;
            /**
             * Event emitted when user change the selected color (without confirm)
             */
            this.change = new core.EventEmitter();
            /**
             * Event emitted when selected color is confirm
             */
            this.selected = new core.EventEmitter();
            /**
             * Event emitted when is clicked outside of the component
             */
            this.clickOut = new core.EventEmitter();
            /**
             * Event emitted when is clicked outside of the component
             */
            this.canceled = new core.EventEmitter();
            /**
             * Array of subscriptions from the collections
             */
            this._collectionSubs = [];
            this._alpha = showAlphaSelector;
            this.format = colorStringFormat;
        }
        Object.defineProperty(MccColorPickerComponent.prototype, "disabled", {
            get: function () {
                return this._disabled;
            },
            /**
             * Disable color-picker of showing up
             */
            set: function (disabled) {
                this._disabled = coercion.coerceBooleanProperty(disabled);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "usedColorLabel", {
            /**
             * Change label of the collection UsedColors
             */
            get: function () {
                return this._usedColorLabel;
            },
            set: function (value) {
                this._usedColorLabel = value;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "usedColorStart", {
            /**
             * Set initial value for used color
             */
            set: function (colors) {
                var e_1, _a;
                if (colors && colors.length > 0) {
                    try {
                        for (var colors_1 = __values(colors), colors_1_1 = colors_1.next(); !colors_1_1.done; colors_1_1 = colors_1.next()) {
                            var color = colors_1_1.value;
                            this.colorPickerService.addColor(color);
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (colors_1_1 && !colors_1_1.done && (_a = colors_1.return)) _a.call(colors_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                }
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "reverseUsedColors", {
            get: function () {
                return this._reverseUsedColor;
            },
            /**
             * Set usedColor to be used in reverse
             */
            set: function (reverse) {
                this._reverseUsedColor = coercion.coerceBooleanProperty(reverse);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "usedColorsPosition", {
            get: function () {
                return this._usedColorsPosition;
            },
            /**
             * Set position of used colors collection
             */
            set: function (position) {
                this._usedColorsPosition = position;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "hideHexForms", {
            get: function () {
                return this._hideHexForms;
            },
            /**
             * Hide the hexadecimal color forms.
             */
            set: function (value) {
                this._hideHexForms = coercion.coerceBooleanProperty(value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "hideEmpty", {
            get: function () {
                return this._hideEmpty;
            },
            /**
             * Hide empty slots from the collection UsedColors
             */
            set: function (value) {
                this._hideEmpty = coercion.coerceBooleanProperty(value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "hideTransparent", {
            get: function () {
                return this._hideTransparent;
            },
            /**
             * Hide transparent option of UsedColors
             */
            set: function (value) {
                this._hideTransparent = coercion.coerceBooleanProperty(value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "hideUsedColors", {
            get: function () {
                return this._hideUsedColors;
            },
            /**
             * Hide UsedColors collection
             */
            set: function (value) {
                this._hideUsedColors = coercion.coerceBooleanProperty(value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "selectedColor", {
            /**
             * Start with a color selected
             */
            get: function () {
                return this._selectedColor;
            },
            set: function (value) {
                if (this._selectedColor !== value) {
                    this.changeDetectorRef.markForCheck();
                }
                var color = parseColorString(value);
                if (color) {
                    this._selectedColor = formatColor(color, this.format);
                }
                else {
                    this._selectedColor = this.emptyColor;
                }
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "isOpen", {
            get: function () {
                return this._isOpen;
            },
            /**
             * Define if the panel will be initiated open
             */
            set: function (value) {
                this._isOpen = coercion.coerceBooleanProperty(value);
                this.changeDetectorRef.detectChanges();
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "overlay", {
            get: function () {
                return this._overlay;
            },
            /**
             * Define if the panel will show in overlay or not
             */
            set: function (value) {
                this._overlay = coercion.coerceBooleanProperty(value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "hideButtons", {
            get: function () {
                return this._hideButtons;
            },
            /**
             * Hide the action buttons (cancel/confirm)
             */
            set: function (value) {
                this._hideButtons = coercion.coerceBooleanProperty(value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "colorPickerSelectorHeight", {
            get: function () {
                return this._colorPickerSelectorHeight;
            },
            /**
             * Define new height for the selector
             */
            set: function (height) {
                this._colorPickerSelectorHeight = height;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "hideColorPickerSelector", {
            get: function () {
                return this._hideColorPickerSelector;
            },
            /**
             * Hide the color picker selector
             */
            set: function (value) {
                this._hideColorPickerSelector = coercion.coerceBooleanProperty(value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "alpha", {
            get: function () {
                return this._alpha;
            },
            /**
             * Show alpha selector
             */
            set: function (value) {
                this._alpha = coercion.coerceBooleanProperty(value);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "tmpSelectedColor$", {
            /**
             * Return a Observable with the color the user is picking
             */
            get: function () {
                return this._tmpSelectedColor.asObservable();
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerComponent.prototype, "usedColors$", {
            /**
             * Observable with all the colors used by the user
             */
            get: function () {
                var _this = this;
                return this.colorPickerService.getUsedColors().pipe(operators.map(function (colors) { return colors.map(function (color) {
                    var clr = parseColorString(color);
                    if (_this._alpha || clr.getAlpha() === 1) {
                        return formatColor(clr, _this.format);
                    }
                }); }), operators.map(function (colors) { return (!_this._reverseUsedColor ? colors : __spread(colors).reverse()); }));
            },
            enumerable: false,
            configurable: true
        });
        MccColorPickerComponent.prototype.ngOnInit = function () {
            this.colorPickerCollectionService.alpha = this._alpha;
            this.colorPickerCollectionService.format = this.format;
            if (!this._selectedColor) {
                this._selectedColor = this.emptyColor;
            }
            this._tmpSelectedColor = new rxjs.BehaviorSubject(this._selectedColor);
        };
        /**
         * Walk throw all collections and subcribe to changes
         */
        MccColorPickerComponent.prototype.ngAfterContentInit = function () {
            var _this = this;
            if (this._collections) {
                this._collections.forEach(function (collection) {
                    var subscription = collection.changeColor.subscribe(function (color) {
                        _this.updateTmpSelectedColor(color);
                    });
                    _this._collectionSubs.push(subscription);
                });
            }
            // change selected color on service
            this.selected.subscribe(function (color) { return _this.colorPickerCollectionService.changeSelectedColor(color); });
        };
        /**
         * Destroy all subscriptions
         */
        MccColorPickerComponent.prototype.ngOnDestroy = function () {
            if (this._collectionSubs) {
                this._collectionSubs.forEach(function (subscription) {
                    if (subscription && !subscription.closed) {
                        subscription.unsubscribe();
                    }
                });
            }
        };
        /**
         * Update selected color and emit the change
         */
        MccColorPickerComponent.prototype._updateSelectedColor = function () {
            if (this._isOpen || !this.overlay) {
                var color = this._tmpSelectedColor.getValue();
                if (this._selectedColor !== color) {
                    this._selectedColor = color;
                    this.selected.emit(color);
                }
                else {
                    this.selected.emit(color);
                }
            }
        };
        /**
         * Open/close color picker panel
         */
        MccColorPickerComponent.prototype.toggle = function () {
            if (!this._disabled) {
                this._isOpen = !this._isOpen;
                if (this._selectedColor !== this.emptyColor) {
                    this.colorPickerService.addColor(this._selectedColor);
                }
                this.colorPickerCollectionService.changeSelectedColor(this._selectedColor);
            }
        };
        /**
         * Update selected color, close the panel and notify the user
         */
        MccColorPickerComponent.prototype.backdropClick = function () {
            if (this._hideButtons) {
                this.confirmSelectedColor();
            }
            else {
                this.cancelSelection();
            }
            this.clickOut.emit();
            this.changeDetectorRef.detectChanges();
        };
        /**
         * Update tmpSelectedColor
         * @param color string
         */
        MccColorPickerComponent.prototype.updateTmpSelectedColor = function (color) {
            var clr = parseColorString(color);
            var clrString = color === this.emptyColor ? color : formatColor(clr, this.format);
            this._tmpSelectedColor.next(clrString);
            if (this._selectedColor !== clrString) {
                this.change.emit(clrString);
                if (this._hideButtons) {
                    this._updateSelectedColor();
                }
            }
        };
        /**
         * Cancel the selection and close the panel
         */
        MccColorPickerComponent.prototype.cancelSelection = function () {
            this._tmpSelectedColor.next(this._selectedColor);
            this.colorPickerCollectionService.changeSelectedColor(this.selectedColor);
            this.canceled.emit();
            this.toggle();
        };
        /**
         * Update selectedColor and close the panel
         */
        MccColorPickerComponent.prototype.confirmSelectedColor = function () {
            this._updateSelectedColor();
            this.toggle();
        };
        return MccColorPickerComponent;
    }());
    MccColorPickerComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'mcc-color-picker',
                    template: "<!-- color picker overlay -->\n<ng-container *ngIf=\"overlay\">\n  <button\n    type=\"button\"\n    class=\"btn-picker\"\n    cdkOverlayOrigin\n    #triggerButton=\"cdkOverlayOrigin\"\n    (click)=\"toggle()\"\n    [ngClass]=\"{ empty: selectedColor === emptyColor, disabled: disabled }\"\n  >\n    <span class=\"btn-picker-background\" [style.backgroundColor]=\"selectedColor\"></span>\n  </button>\n\n<!--  TODO: try to center with the overlay centerHorizontally on mobile instead of cdkConnectedOverlayPush-->\n  <ng-template\n    cdkConnectedOverlay\n    cdkConnectedOverlayHasBackdrop\n    cdkConnectedOverlayBackdropClass=\"mcc-color-picker-backdrop\"\n    cdkConnectedOverlayPush\n    [cdkConnectedOverlayOrigin]=\"triggerButton\"\n    [cdkConnectedOverlayOpen]=\"isOpen\"\n    (backdropClick)=\"backdropClick()\"\n  >\n    <ng-template [cdkPortalOutlet]=\"overlayPanel\"></ng-template>\n  </ng-template>\n</ng-container>\n\n<!-- color picker flat -->\n<ng-template *ngIf=\"!overlay\" [cdkPortalOutlet]=\"overlayPanel\"></ng-template>\n\n<!-- color picker component content -->\n<ng-template cdkPortal #overlayPanel=\"cdkPortal\">\n  <div class=\"mcc-color-picker-overlay mat-elevation-z4\" [ngClass]=\"{ 'alpha-enabled': alpha }\" role=\"dialog\" aria-label=\"Color picker\">\n    <mcc-color-picker-selector\n      *ngIf=\"!hideColorPickerSelector\"\n      [selectedColor]=\"tmpSelectedColor$ | async\"\n      [hideHexForms]=\"hideHexForms\"\n      [height]=\"colorPickerSelectorHeight\"\n      (changeSelectedColor)=\"updateTmpSelectedColor($event)\"\n    >\n    </mcc-color-picker-selector>\n\n    <!-- show used colors on top -->\n    <ng-container *ngIf=\"!hideUsedColors && usedColorsPosition == 'top'\">\n      <mcc-color-picker-collection\n        [label]=\"usedColorLabel\"\n        [size]=\"usedSizeColors\"\n        [transparent]=\"!hideTransparent\"\n        [hideEmpty]=\"hideEmpty\"\n        [colors]=\"usedColors$ | async\"\n        (changeColor)=\"updateTmpSelectedColor($event)\"\n      >\n      </mcc-color-picker-collection>\n    </ng-container>\n\n    <!-- render user collections -->\n    <ng-content></ng-content>\n\n    <!-- show used colors on bottom -->\n    <ng-container *ngIf=\"!hideUsedColors && usedColorsPosition == 'bottom'\">\n      <mcc-color-picker-collection\n        [label]=\"usedColorLabel\"\n        [size]=\"usedSizeColors\"\n        [transparent]=\"!hideTransparent\"\n        [hideEmpty]=\"hideEmpty\"\n        [colors]=\"usedColors$ | async\"\n        (changeColor)=\"updateTmpSelectedColor($event)\"\n      >\n      </mcc-color-picker-collection>\n    </ng-container>\n\n    <div *ngIf=\"!hideButtons\" class=\"mcc-color-picker-actions\">\n      <div style=\"flex: 1 auto;\"></div>\n      <button mat-button role=\"button\" aria-label=\"Cancel\" (click)=\"cancelSelection()\">\n        {{ btnCancelLabel }}\n      </button>\n\n      <button mat-button role=\"button\" aria-label=\"Confirm\" (click)=\"confirmSelectedColor()\">\n        {{ btnConfirmLabel }}\n      </button>\n    </div>\n  </div>\n</ng-template>\n",
                    preserveWhitespaces: false,
                    changeDetection: core.ChangeDetectionStrategy.OnPush,
                    providers: [MccColorPickerCollectionService],
                    styles: [".btn-picker{background-image:url('data:image/svg+xml;charset=utf-8,<svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 2 2\"><path d=\"M1 2V0h1v1H0v1z\" fill-opacity=\"0.1\"/></svg>');background-repeat:repeat;background-size:14px;border:2px solid #ddd;border-radius:3px;cursor:pointer;height:25px;position:relative;width:25px}.btn-picker .btn-picker-background{bottom:0;left:0;position:absolute;right:0;top:0}.btn-picker.empty{background-image:url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAB8AAAAfAQMAAAAlYIR6AAAABlBMVEX/////EBC8L+WmAAAAJ0lEQVR42mM4wAAED0DEBxBRASJsQIQciOAHEewNQIIZpJCRjgr5AP7LD2UBZrkqAAAAAElFTkSuQmCC\")!important;background-repeat:no-repeat;background-size:auto}.btn-picker.disabled{opacity:.5}.mcc-color-picker-overlay{background:#fff;border-radius:4px;display:flex;flex-direction:column;font-family:Roboto,sans-serif;min-height:80px;padding:0;position:relative;width:270px}.mcc-color-picker-overlay.alpha-enabled{width:316px}.mcc-color-picker-overlay .mcc-color-picker-preview{height:8px;width:100%}.transparent{-webkit-transform:translateY(-2px) translateX(-11px) rotate(45deg);border-bottom:2px solid red;display:block;height:2px;position:absolute;transform:translateY(-3px) translateX(-2px) rotate(45deg);width:32px}.mcc-color-picker-actions{border-top:1px solid #ddd;display:flex;padding:4px}.mcc-color-picker-actions button{text-transform:uppercase}::ng-deep mat-form-field mcc-color-picker .btn-picker{margin-left:4px;margin-right:4px}"]
                },] }
    ];
    MccColorPickerComponent.ctorParameters = function () { return [
        { type: core.ElementRef },
        { type: core.ChangeDetectorRef },
        { type: MccColorPickerService },
        { type: MccColorPickerCollectionService },
        { type: String, decorators: [{ type: core.Inject, args: [EMPTY_COLOR,] }] },
        { type: Boolean, decorators: [{ type: core.Inject, args: [ENABLE_ALPHA_SELECTOR,] }] },
        { type: undefined, decorators: [{ type: core.Inject, args: [COLOR_STRING_FORMAT,] }] }
    ]; };
    MccColorPickerComponent.propDecorators = {
        _collections: [{ type: core.ContentChildren, args: [MccColorPickerCollectionComponent,] }],
        disabled: [{ type: core.Input }],
        usedColorLabel: [{ type: core.Input }],
        usedColorStart: [{ type: core.Input }],
        reverseUsedColors: [{ type: core.Input }],
        usedColorsPosition: [{ type: core.Input }],
        hideHexForms: [{ type: core.Input }],
        hideEmpty: [{ type: core.Input, args: ['hideEmptyUsedColors',] }],
        hideTransparent: [{ type: core.Input, args: ['hideTransparentUsedColors',] }],
        hideUsedColors: [{ type: core.Input, args: ['hideUsedColors',] }],
        selectedColor: [{ type: core.Input }],
        isOpen: [{ type: core.Input }],
        overlay: [{ type: core.Input }],
        hideButtons: [{ type: core.Input }],
        colorPickerSelectorHeight: [{ type: core.Input }],
        hideColorPickerSelector: [{ type: core.Input }],
        usedSizeColors: [{ type: core.Input }],
        btnCancelLabel: [{ type: core.Input }],
        btnConfirmLabel: [{ type: core.Input }],
        alpha: [{ type: core.Input }],
        format: [{ type: core.Input }],
        change: [{ type: core.Output }],
        selected: [{ type: core.Output }],
        clickOut: [{ type: core.Output }],
        canceled: [{ type: core.Output }]
    };

    /**
     * Directive connect an color picker with any input, select or textarea.
     * The color picker will be automatically updated when the value of the origin is
     * changed.
     */
    var MccConnectedColorPickerDirective = /** @class */ (function () {
        function MccConnectedColorPickerDirective(colorPicker, changeDetectorRef, emptyColor) {
            this.colorPicker = colorPicker;
            this.changeDetectorRef = changeDetectorRef;
            this.emptyColor = emptyColor;
        }
        MccConnectedColorPickerDirective.prototype.ngAfterViewInit = function () {
            if (!this._colorPickerSub) {
                this._attachColorPicker();
            }
        };
        MccConnectedColorPickerDirective.prototype.ngOnDestroy = function () {
            if (this._colorPickerSub && !this._colorPickerSub.closed) {
                this._colorPickerSub.unsubscribe();
            }
            if (this._originSub && !this._originSub.closed) {
                this._originSub.unsubscribe();
            }
            if (this._focusSub && !this._focusSub.closed) {
                this._focusSub.unsubscribe();
            }
        };
        /**
         * Attach color picker and origin
         */
        MccConnectedColorPickerDirective.prototype._attachColorPicker = function () {
            var _this = this;
            // subscribe to origin change to update color picker
            this._originSub = this.origin.change.subscribe(function (value) {
                var color = parseColorString(value);
                if (value === _this.emptyColor && _this.colorPicker.selectedColor !== _this.emptyColor) {
                    _this.colorPicker.updateTmpSelectedColor(value);
                }
                else if (color) {
                    _this.colorPicker.updateTmpSelectedColor(value);
                    _this.colorPicker.selectedColor = value;
                    _this.changeDetectorRef.detectChanges();
                }
            });
            this._focusSub = this.origin.hasFocus.subscribe(function () {
                _this.colorPicker.isOpen = true;
            });
            // subscribe to color picker confirmation and set on origin element
            // TODO: changed to on confirm. maybe support on-change mode again?
            this._colorPickerSub = this.colorPicker.selected.subscribe(function (value) { return _this.origin.writeValueFromColorPicker(value); });
        };
        return MccConnectedColorPickerDirective;
    }());
    MccConnectedColorPickerDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[mcc-connected-color-picker], [mccConnectedColorPicker]',
                    exportAs: 'mccConnectedColorPicker'
                },] }
    ];
    MccConnectedColorPickerDirective.ctorParameters = function () { return [
        { type: MccColorPickerComponent },
        { type: core.ChangeDetectorRef },
        { type: String, decorators: [{ type: core.Inject, args: [EMPTY_COLOR,] }] }
    ]; };
    MccConnectedColorPickerDirective.propDecorators = {
        origin: [{ type: core.Input, args: ['mccConnectedColorPickerOrigin',] }]
    };

    var INITIAL_COLOR = new esTinycolor.TinyColor('white');
    var MccColorPickerSelectorComponent = /** @class */ (function () {
        function MccColorPickerSelectorComponent(formBuilder, renderer, colorPickerCollectionService, emptyColor) {
            this.formBuilder = formBuilder;
            this.renderer = renderer;
            this.colorPickerCollectionService = colorPickerCollectionService;
            this.emptyColor = emptyColor;
            this._height = 170;
            this._selectedColor = INITIAL_COLOR;
            this._hideHexForms = false;
            /**
             * Emit update when a color is selected
             */
            this.changeSelectedColor = new core.EventEmitter();
            /**
             * Subject of the current selected color by the user
             */
            this._tmpSelectedColor = new rxjs.BehaviorSubject(this._selectedColor);
            /**
             * Handle color of the form inputs text
             */
            this.textClass = 'black';
            /**
             * Keeps track if the mouse button is pressed
             */
            this._isPressed = false;
            /**
             * keeping mousedown listeners for destruction
             */
            this._mouseDownListeners = [];
            /**
             * keeping track of temporary mouse listeners for destruction
             */
            this._temporaryMouseListeners = [];
            this._selectorWidth = 240;
            this._stripWidth = 20;
        }
        Object.defineProperty(MccColorPickerSelectorComponent.prototype, "hueContainer", {
            /**
             * Container of the strip
             */
            set: function (el) {
                this._sc = el;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerSelectorComponent.prototype, "height", {
            get: function () {
                return this._height;
            },
            /**
             * Change height base of the selector
             */
            set: function (value) {
                this._height = value;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerSelectorComponent.prototype, "stripHeight", {
            get: function () {
                return this._height - 10;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerSelectorComponent.prototype, "selectedColor", {
            /**
             * Receive selected color from the component
             */
            set: function (value) {
                if (value === this.emptyColor) {
                    this.noColor = true;
                    // use "EMPTY_COLOR" as real color if it can be parsed as such
                    this._selectedColor = parseColorString(this.emptyColor) || INITIAL_COLOR;
                }
                else {
                    var color = parseColorString(value);
                    if (color) {
                        this._selectedColor = color.setAlpha(color['_roundA']);
                        this.noColor = false;
                    }
                }
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerSelectorComponent.prototype, "hideHexForms", {
            get: function () {
                return this._hideHexForms;
            },
            /**
             * Hide the hexadecimal color forms.
             */
            set: function (value) {
                this._hideHexForms = value;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerSelectorComponent.prototype, "selectorWidth", {
            get: function () {
                return this._selectorWidth;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerSelectorComponent.prototype, "stripWidth", {
            get: function () {
                return this._stripWidth;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerSelectorComponent.prototype, "tmpSelectedColorAsRgba$", {
            get: function () {
                return this._tmpSelectedColor.asObservable().pipe(operators.map(toRgb), operators.distinctUntilChanged());
            },
            enumerable: false,
            configurable: true
        });
        MccColorPickerSelectorComponent.prototype.ngOnInit = function () {
            var _this = this;
            // set initial position for selection block
            this._latestBlockCoordinates = { x: this._selectorWidth - 1, y: Math.floor(this._height / 2) };
            // set selectedColor initial value
            this._tmpSelectedColor.next(this._selectedColor);
            this._tmpSelectedColorSub = this._tmpSelectedColor.subscribe(function (color) {
                _this.textClass = color.isDark() && color.getAlpha() > 0.3 ? 'white' : 'black';
                // right now using hex for non alpha and rgba for alpha colors
                if (_this.noColor) {
                    _this.changeSelectedColor.emit(_this.emptyColor);
                    _this.colorPickerCollectionService.changeSelectedColor(_this.emptyColor);
                }
                else {
                    var clr = formatColor(color, _this.colorPickerCollectionService.format);
                    _this.changeSelectedColor.emit(clr);
                    _this.colorPickerCollectionService.changeSelectedColor(clr);
                }
            });
            // hex form
            this.hexForm = this.formBuilder.group({
                hexCode: new forms.FormControl(toHex(this._selectedColor), {
                    validators: [forms.Validators.pattern(/^#([0-9A-F]{8}|[0-9A-F]{6})$/gi)],
                    updateOn: 'change'
                })
            });
            // rgb form
            var rgbKeys = ['R', 'G', 'B'];
            var rgbaGroup = {};
            var rgb = this._selectedColor.toRgb();
            var rgbValues = [rgb.r, rgb.g, rgb.b, rgb.a];
            rgbKeys.forEach(function (key, index) { return (rgbaGroup[key] = new forms.FormControl(rgbValues[index], {
                validators: [forms.Validators.min(0), forms.Validators.max(255), forms.Validators.required],
                updateOn: 'change'
            })); });
            // add alpha input to rgb form
            rgbaGroup['A'] = new forms.FormControl(this._selectedColor.getAlpha(), {
                validators: [forms.Validators.min(0), forms.Validators.max(1), forms.Validators.required]
            });
            this.rgbaForm = this.formBuilder.group(rgbaGroup);
            this._watchFormChanges();
        };
        /**
         * Update forms and selectors selectedColor change and
         * the mouse button is not pressed
         */
        MccColorPickerSelectorComponent.prototype.ngOnChanges = function (changes) {
            if ('selectedColor' in changes) {
                if (!this._isPressed) {
                    var color = this._selectedColor;
                    this._updateHexForm(color);
                    this._updateRGBAForm(color);
                    this.setSelectorPositions(color);
                    this._drawBlockSelector(color);
                    if (this.colorPickerCollectionService.alpha) {
                        this._drawAlphaSelector(color);
                    }
                    this._tmpSelectedColor.next(color);
                }
            }
        };
        MccColorPickerSelectorComponent.prototype.ngOnDestroy = function () {
            // destroy listeners
            this._mouseDownListeners.forEach(function (listener) { return listener(); });
            this._temporaryMouseListeners.forEach(function (listener) { return listener(); });
            // unsubscribe
            if (this._tmpSelectedColorSub && !this._tmpSelectedColorSub.closed) {
                this._tmpSelectedColorSub.unsubscribe();
            }
            if (this._hexValuesSub && !this._hexValuesSub.closed) {
                this._hexValuesSub.unsubscribe();
            }
            if (this._rgbValuesSub && !this._rgbValuesSub.closed) {
                this._rgbValuesSub.unsubscribe();
            }
        };
        MccColorPickerSelectorComponent.prototype.ngAfterViewInit = function () {
            var _this = this;
            // draw initial selectors and listen to mouse events
            this._mouseDownListeners.push(this.renderer.listen(this._block.nativeElement, 'touchstart', function (e) {
                if (e.cancelable) {
                    e.preventDefault();
                    _this._isPressed = true;
                    _this._selectionTargetPosition = _this._block.nativeElement.getBoundingClientRect();
                    _this._listenToMouseForBlock();
                    var x = e.targetTouches[0].clientX - _this._selectionTargetPosition.x;
                    var y = e.targetTouches[0].clientY - _this._selectionTargetPosition.y;
                    _this.changeColor({ x: x, y: y });
                }
            }));
            this._mouseDownListeners.push(this.renderer.listen(this._block.nativeElement, 'mousedown', function (e) {
                e.preventDefault();
                _this._isPressed = true;
                _this._selectionTargetPosition = _this._block.nativeElement.getBoundingClientRect();
                _this._listenToMouseForBlock();
                _this.changeColor({ x: e.offsetX, y: e.offsetY });
            }));
            this._blockContext = this._bc.nativeElement.getContext('2d');
            this._blockContext.rect(0, 0, this._bc.nativeElement.width, this._bc.nativeElement.height);
            this._mouseDownListeners.push(this.renderer.listen(this._hueSelector.nativeElement, 'touchstart', function (e) {
                if (e.cancelable) {
                    e.preventDefault();
                    _this._isPressed = true;
                    _this._selectionTargetPosition = _this._hueSelector.nativeElement.getBoundingClientRect();
                    _this._listenToMouseForHue();
                    var y = e.targetTouches[0].clientY - _this._selectionTargetPosition.y;
                    _this.changeHue(y);
                }
            }));
            this._mouseDownListeners.push(this.renderer.listen(this._hueSelector.nativeElement, 'mousedown', function (e) {
                e.preventDefault();
                _this._isPressed = true;
                _this._selectionTargetPosition = _this._hueSelector.nativeElement.getBoundingClientRect();
                _this._listenToMouseForHue();
                _this.changeHue(e.offsetY);
            }));
            if (this.colorPickerCollectionService.alpha) {
                this._drawAlphaSelector(this._selectedColor);
                this._mouseDownListeners.push(this.renderer.listen(this._alpha.nativeElement, 'touchstart', function (e) {
                    if (e.cancelable) {
                        e.preventDefault();
                        _this._isPressed = true;
                        _this._selectionTargetPosition = _this._alpha.nativeElement.getBoundingClientRect();
                        _this._listenToMouseForAlpha();
                        var y = e.targetTouches[0].clientY - _this._selectionTargetPosition.y;
                        _this.changeAlpha(y);
                    }
                }));
                this._mouseDownListeners.push(this.renderer.listen(this._alpha.nativeElement, 'mousedown', function (e) {
                    e.preventDefault();
                    _this._isPressed = true;
                    _this._selectionTargetPosition = _this._alpha.nativeElement.getBoundingClientRect();
                    _this._listenToMouseForAlpha();
                    _this.changeAlpha(e.offsetY);
                }));
                this._alphaContext = this._alpha.nativeElement.getContext('2d');
            }
            this._drawHueSelector();
            this._drawBlockSelector(this._selectedColor);
            if (this.colorPickerCollectionService.alpha) {
                this._drawAlphaSelector(this._selectedColor);
            }
            this.setSelectorPositions(this._selectedColor);
        };
        MccColorPickerSelectorComponent.prototype._listenToMouseForBlock = function () {
            var _this = this;
            this._preventTextSelection();
            this._temporaryMouseListeners.push(this.renderer.listen(this._block.nativeElement, 'touchmove', function (e) {
                if (e.cancelable) {
                    e.preventDefault();
                    var coords = _this._getCoordinatesForPosition(e.targetTouches[0].clientX, e.targetTouches[0].clientY);
                    _this.changeColor(coords);
                }
            }));
            this._temporaryMouseListeners.push(this.renderer.listen(this._block.nativeElement, 'mousemove', function (e) {
                if (e.buttons === 1) {
                    _this.changeColor({ x: e.offsetX, y: e.offsetY });
                }
            }));
            this._registerMouseUpListeners();
            this._temporaryMouseListeners.push(this.renderer.listen(document, 'mousemove', function (e) {
                if (_this._isPressed && e.target !== _this._block.nativeElement) {
                    var coords = _this._getCoordinatesForPosition(e.clientX, e.clientY);
                    _this.changeColor(coords);
                }
            }));
        };
        MccColorPickerSelectorComponent.prototype._listenToMouseForHue = function () {
            var _this = this;
            this._preventTextSelection();
            this._temporaryMouseListeners.push(this.renderer.listen(this._hueSelector.nativeElement, 'touchmove', function (e) {
                if (e.cancelable) {
                    e.preventDefault();
                    var y = _this._getYForPosition(e.targetTouches[0].clientY);
                    _this.changeHue(y);
                }
            }));
            this._temporaryMouseListeners.push(this.renderer.listen(this._hueSelector.nativeElement, 'mousemove', function (e) {
                if (e.buttons === 1) {
                    _this.changeHue(e.offsetY);
                }
            }));
            this._registerMouseUpListeners();
            this._temporaryMouseListeners.push(this.renderer.listen(document, 'mousemove', function (e) {
                if (_this._isPressed && e.target !== _this._hueSelector.nativeElement) {
                    var y = _this._getYForPosition(e.clientY);
                    _this.changeHue(y);
                }
            }));
        };
        MccColorPickerSelectorComponent.prototype._listenToMouseForAlpha = function () {
            var _this = this;
            this._preventTextSelection();
            this._temporaryMouseListeners.push(this.renderer.listen(this._alpha.nativeElement, 'touchmove', function (e) {
                if (e.cancelable) {
                    e.preventDefault();
                    var y = _this._getYForPosition(e.targetTouches[0].clientY);
                    _this.changeAlpha(y);
                }
            }));
            this._temporaryMouseListeners.push(this.renderer.listen(this._alpha.nativeElement, 'mousemove', function (e) {
                if (e.buttons === 1) {
                    _this.changeAlpha(e.offsetY);
                }
            }));
            this._registerMouseUpListeners();
            this._temporaryMouseListeners.push(this.renderer.listen(document, 'mousemove', function (e) {
                if (_this._isPressed && e.target !== _this._alpha.nativeElement) {
                    var y = _this._getYForPosition(e.clientY);
                    _this.changeAlpha(y);
                }
            }));
        };
        MccColorPickerSelectorComponent.prototype._preventTextSelection = function () {
            this._temporaryMouseListeners.push(this.renderer.listen(document, 'selectstart', function (e) {
                e.preventDefault();
            }));
        };
        MccColorPickerSelectorComponent.prototype._registerMouseUpListeners = function () {
            var _this = this;
            this._temporaryMouseListeners.push(this.renderer.listen(document, 'touchend', function (e) {
                if (e.cancelable) {
                    e.preventDefault();
                    _this._isPressed = false;
                    _this._temporaryMouseListeners.forEach(function (listener) { return listener(); });
                }
            }));
            this._temporaryMouseListeners.push(this.renderer.listen(document, 'mouseup', function (e) {
                e.preventDefault();
                _this._isPressed = false;
                _this._temporaryMouseListeners.forEach(function (listener) { return listener(); });
            }));
        };
        MccColorPickerSelectorComponent.prototype._getCoordinatesForPosition = function (clientX, clientY) {
            var x;
            if (clientX < this._selectionTargetPosition.left) {
                x = 0;
            }
            else if (clientX > this._selectionTargetPosition.right) {
                x = this._selectionTargetPosition.width;
            }
            else {
                x = clientX - this._selectionTargetPosition.left;
            }
            var y;
            if (clientY < this._selectionTargetPosition.top) {
                y = 0;
            }
            else if (clientY > this._selectionTargetPosition.bottom) {
                y = this._selectionTargetPosition.height;
            }
            else {
                y = clientY - this._selectionTargetPosition.top;
            }
            return { x: x, y: y };
        };
        MccColorPickerSelectorComponent.prototype._getYForPosition = function (clientY) {
            var y;
            if (clientY < this._selectionTargetPosition.top) {
                y = 0;
            }
            else if (clientY > this._selectionTargetPosition.bottom) {
                y = this._selectionTargetPosition.height;
            }
            else {
                y = clientY - this._selectionTargetPosition.top;
            }
            return y;
        };
        MccColorPickerSelectorComponent.prototype._drawHueSelector = function () {
            this._hueSelectorContext = this._hueSelector.nativeElement.getContext('2d');
            this._hueSelectorContext.rect(0, 0, this._hueSelector.nativeElement.width, this._hueSelector.nativeElement.height);
            var grd1 = this._hueSelectorContext.createLinearGradient(0, 0, 0, this._bc.nativeElement.height);
            grd1.addColorStop(0, 'hsl(0, 100%, 50%)');
            grd1.addColorStop(0.125, 'hsl(45, 100%, 50%)');
            grd1.addColorStop(0.25, 'hsl(90, 100%, 50%)');
            grd1.addColorStop(0.375, 'hsl(135, 100%, 50%)');
            grd1.addColorStop(0.5, 'hsl(180, 100%, 50%)');
            grd1.addColorStop(0.625, 'hsl(225, 100%, 50%)');
            grd1.addColorStop(0.75, 'hsl(270, 100%, 50%)');
            grd1.addColorStop(0.875, 'hsl(315, 100%, 50%)');
            grd1.addColorStop(1, 'hsl(360, 100%, 50%)');
            this._hueSelectorContext.fillStyle = grd1;
            this._hueSelectorContext.fill();
        };
        /**
         * Generate alpha selector gradient based on the RGB color
         */
        MccColorPickerSelectorComponent.prototype._drawAlphaSelector = function (color) {
            if (this._alphaContext) {
                this._alphaContext.clearRect(0, 0, this._alpha.nativeElement.width, this._alpha.nativeElement.height);
                var alphaGrd2 = this._alphaContext.createLinearGradient(0, 0, 0, this._bc.nativeElement.height);
                alphaGrd2.addColorStop(0, "rgba(" + color.toRgb().r + ", " + color.toRgb().g + ", " + color.toRgb().b + ", 1)");
                alphaGrd2.addColorStop(1, "rgba(" + color.toRgb().r + ", " + color.toRgb().g + ", " + color.toRgb().b + ", 0)");
                this._alphaContext.fillStyle = alphaGrd2;
                this._alphaContext.fillRect(0, 0, this._alpha.nativeElement.width, this._alpha.nativeElement.height);
            }
        };
        /**
         * Generate color selector block based on the RGB color
         */
        MccColorPickerSelectorComponent.prototype._drawBlockSelector = function (hueColor) {
            if (this._blockContext) {
                this._blockContext.fillStyle = "hsl(" + hueColor.toHsl().h + ", 100%, 50%)";
                this._blockContext.fillRect(0, 0, this._bc.nativeElement.width, this._bc.nativeElement.height);
                // HSL selector
                var grdWhite = this._hueSelectorContext.createLinearGradient(1, 0, this._bc.nativeElement.width - 1, 0);
                grdWhite.addColorStop(0, 'rgba(255,255,255,1)');
                grdWhite.addColorStop(1, 'rgba(255,255,255,0)');
                this._blockContext.fillStyle = grdWhite;
                this._blockContext.fillRect(0, 0, this._bc.nativeElement.width, this._bc.nativeElement.height);
                var grdBlack = this._hueSelectorContext.createLinearGradient(0, 1, 0, this._bc.nativeElement.height - 1);
                grdBlack.addColorStop(0, 'rgba(0,0,0,0)');
                grdBlack.addColorStop(1, 'rgba(0,0,0,1)');
                this._blockContext.fillStyle = grdBlack;
                this._blockContext.fillRect(0, 0, this._bc.nativeElement.width, this._bc.nativeElement.height);
            }
        };
        /**
         * Watch changes on forms
         */
        MccColorPickerSelectorComponent.prototype._watchFormChanges = function () {
            var _this = this;
            this._hexValuesSub = this.hexForm.valueChanges.subscribe(function (value) {
                if (_this.hexForm.valid) {
                    var color = new esTinycolor.TinyColor(value.hexCode);
                    _this._updateRGBAForm(color);
                    _this._drawBlockSelector(color);
                    if (_this.colorPickerCollectionService.alpha) {
                        _this._drawAlphaSelector(color);
                    }
                    _this.setSelectorPositions(color);
                    _this._tmpSelectedColor.next(color);
                }
            });
            this._rgbValuesSub = this.rgbaForm.valueChanges.subscribe(function (rgba) {
                if (_this.rgbaForm.valid) {
                    var color = new esTinycolor.TinyColor({ r: rgba.R, g: rgba.G, b: rgba.B, a: rgba.A });
                    _this._updateHexForm(color);
                    _this._drawBlockSelector(color);
                    if (_this.colorPickerCollectionService.alpha) {
                        _this._drawAlphaSelector(color);
                    }
                    _this.setSelectorPositions(color);
                    _this._tmpSelectedColor.next(color);
                }
            });
        };
        /**
         * Update RGBA form
         */
        MccColorPickerSelectorComponent.prototype._updateRGBAForm = function (color) {
            if (!this.rgbaForm) {
                return;
            }
            var values = { R: color.toRgb().r, G: color.toRgb().g, B: color.toRgb().b, A: color.toRgb().a };
            this.rgbaForm.setValue(values, { emitEvent: false });
        };
        /**
         * Update hex form
         */
        MccColorPickerSelectorComponent.prototype._updateHexForm = function (color) {
            if (!this.hexForm) {
                return;
            }
            var hex = toHex(color);
            var control = this.hexForm.get('hexCode');
            control.setValue(hex, { emitEvent: false, onlySelf: true });
            control.updateValueAndValidity({ emitEvent: false, onlySelf: true });
        };
        /**
         * Handle changes of the hue slider
         */
        MccColorPickerSelectorComponent.prototype.changeHue = function (y) {
            if (y <= this.stripHeight) {
                this.setHueSelector(y);
                var data = this._hueSelectorContext.getImageData(this.stripWidth / 2, y, 1, 1).data;
                var color = new esTinycolor.TinyColor({ r: data[0], g: data[1], b: data[2] });
                this._drawBlockSelector(color);
                this.changeColor();
            }
        };
        /**
         * Handle changes of the alpha slider
         */
        MccColorPickerSelectorComponent.prototype.changeAlpha = function (y) {
            if (y <= this.stripHeight) {
                this.setAlphaSelector(y);
                var alpha = Number(((this.stripHeight - y) / this.stripHeight).toFixed(2));
                var color = this._selectedColor.clone().setAlpha(alpha);
                this._updateRGBAForm(color);
                this._updateHexForm(color);
                this._tmpSelectedColor.next(color);
                this.noColor = false;
            }
        };
        /**
         * Handle changes of the selected color in the XY block
         */
        MccColorPickerSelectorComponent.prototype.changeColor = function (offsets) {
            var os = offsets || this._latestBlockCoordinates;
            if (os.x <= this._selectorWidth && os.y <= this._height) {
                this.setXYSelector(os);
                var data = this._blockContext.getImageData(os.x ? os.x - 1 : os.x, os.y ? os.y - 1 : os.y, 1, 1).data;
                var color = new esTinycolor.TinyColor({ r: data[0], g: data[1], b: data[2], a: this._selectedColor.getAlpha() });
                this._updateRGBAForm(color);
                this._updateHexForm(color);
                if (this.colorPickerCollectionService.alpha) {
                    this._drawAlphaSelector(color);
                }
                this._tmpSelectedColor.next(color);
                this.noColor = false;
            }
        };
        /**
         * Set all selectors positions based on a color
         */
        MccColorPickerSelectorComponent.prototype.setSelectorPositions = function (color) {
            var offset = this.getHueOffsets(color);
            this.setHueSelector(offset);
            var offsets = this.getXYOffsets(color);
            this.setXYSelector(offsets);
            var alphaOffset = this.getAlphaOffset(color);
            this.setAlphaSelector(alphaOffset);
        };
        /**
         * Set block selector positions in view
         */
        MccColorPickerSelectorComponent.prototype.setXYSelector = function (offsets) {
            if (this._bp) {
                this.renderer.setStyle(this._bp.nativeElement, 'transform', "translate(" + (offsets.x - 6) + "px, " + (offsets.y - 6) + "px)");
                this._latestBlockCoordinates = offsets;
            }
        };
        /**
         * Set hue selector positions in view
         */
        MccColorPickerSelectorComponent.prototype.setHueSelector = function (offset) {
            if (this._sc) {
                this.renderer.setStyle(this._sc.nativeElement, 'background-position-y', offset + "px");
            }
        };
        /**
         * Set alpha selector positions in view
         */
        MccColorPickerSelectorComponent.prototype.setAlphaSelector = function (offset) {
            if (this._ac) {
                this.renderer.setStyle(this._ac.nativeElement, 'background-position-y', offset + "px");
            }
        };
        /**
         * Get the X and Y coordinates for the block selector relative to it's size
         */
        MccColorPickerSelectorComponent.prototype.getXYOffsets = function (color) {
            var hsv = color.toHsv();
            var x = this._selectorWidth * hsv.s;
            var y = this._height - this._height * hsv.v;
            return { x: x, y: y };
        };
        /**
         * Get the Y coordinate for the hue selector relative to it's height
         */
        MccColorPickerSelectorComponent.prototype.getHueOffsets = function (color) {
            return (this.stripHeight / 360) * color.toHsl().h;
        };
        /**
         * Get the Y coordinate for the alpha selector relative to it's height
         */
        MccColorPickerSelectorComponent.prototype.getAlphaOffset = function (color) {
            return this.stripHeight - this.stripHeight * color.getAlpha();
        };
        return MccColorPickerSelectorComponent;
    }());
    MccColorPickerSelectorComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'mcc-color-picker-selector',
                    template: "<div class=\"mcc-color-picker-selector\" [ngStyle]=\"{ 'height.px': height }\">\n  <div #block class=\"mcc-picker-selector\" [style.height.px]=\"height\" [style.width.px]=\"selectorWidth\"></div>\n  <canvas #blockCanvas [height]=\"height\" [width]=\"selectorWidth\" id=\"picker\"></canvas>\n  <div #blockPointer class=\"mcc-picker-position\" style=\"top: 0; left: 0;\"></div>\n\n  <div\n    #hueContainer\n    class=\"mcc-colors-position\"\n    [style.height.px]=\"height\"\n    [ngClass]=\"{ 'alpha-enabled': colorPickerCollectionService.alpha }\"\n    style=\"background-position-y: 0;\"\n  >\n    <canvas #hueSelector [height]=\"stripHeight\" [width]=\"stripWidth\" id=\"colors\"></canvas>\n  </div>\n\n  <div\n    *ngIf=\"colorPickerCollectionService.alpha\"\n    #alphaContainer\n    class=\"mcc-alpha-position\"\n    [style.height.px]=\"height\"\n    style=\"background-position-y: 0;\"\n  >\n    <canvas #alpha [height]=\"stripHeight\" [width]=\"stripWidth\" id=\"alpha\" class=\"mcc-alpha-background\"></canvas>\n  </div>\n</div>\n\n<div class=\"mcc-color-picker-selector-preview\">\n  <div class=\"mcc-color-picker-selector-preview-background\" [style.backgroundColor]=\"tmpSelectedColorAsRgba$ | async\"></div>\n  <!--  TODO: refactor hex-input and rgb-input classes to be compatible with appearance=\"standard/outline/fill\"-->\n  <ng-container *ngIf=\"!hideHexForms && !noColor\">\n    <form [formGroup]=\"hexForm\">\n      <mat-form-field class=\"hex-input\" floatLabel=\"always\" appearance=\"legacy\" [ngClass]=\"textClass\">\n        <input matInput placeholder=\"HEX\" formControlName=\"hexCode\" />\n      </mat-form-field>\n    </form>\n\n    <form [formGroup]=\"rgbaForm\">\n      <mat-form-field class=\"rgb-input\" floatLabel=\"always\" appearance=\"legacy\" [ngClass]=\"textClass\">\n        <input matInput type=\"number\" [placeholder]=\"colorPickerCollectionService.alpha ? 'RGBA' : 'RGB'\" formControlName=\"R\" />\n      </mat-form-field>\n      <mat-form-field class=\"rgb-input\" floatLabel=\"always\" appearance=\"legacy\" [ngClass]=\"textClass\">\n        <input matInput type=\"number\" formControlName=\"G\" />\n      </mat-form-field>\n      <mat-form-field class=\"rgb-input\" floatLabel=\"always\" appearance=\"legacy\" [ngClass]=\"textClass\">\n        <input matInput type=\"number\" formControlName=\"B\" />\n      </mat-form-field>\n\n      <mat-form-field\n        *ngIf=\"colorPickerCollectionService.alpha\"\n        class=\"rgb-input\"\n        floatLabel=\"always\"\n        appearance=\"legacy\"\n        [ngClass]=\"textClass\"\n      >\n        <input matInput type=\"number\" formControlName=\"A\" />\n      </mat-form-field>\n    </form>\n  </ng-container>\n</div>\n",
                    preserveWhitespaces: false,
                    changeDetection: core.ChangeDetectionStrategy.OnPush,
                    styles: ["canvas#alpha,canvas#colors{margin:5px}canvas:hover{cursor:crosshair}.mcc-picker-selector{cursor:crosshair;left:0;position:absolute;top:0;z-index:10}.mcc-picker-position{background:rgba(0,0,0,.3);border:1px solid #ddd;border-radius:50%;height:10px;position:absolute;width:10px;z-index:1}.mcc-colors-position{background-image:url(\"data:image/gif;base64,R0lGODdhHgAIALMAAAAAADU1NTk5OUJCQkpKSlZWVltbW2pqaoCAgP///wAAAAAAAAAAAAAAAAAAAAAAACH5BAkAAAoALAAAAAAeAAgAAAQw0BhFq734yjJm/p8xFEVAeGCqGERACmahgkUrvPH81cFdnjoQy8VBBTOiguSokkQAADs=\");background-repeat:no-repeat;position:absolute;right:0;top:0;width:30px;z-index:1}.mcc-colors-position.alpha-enabled{right:40px}.mcc-alpha-position{background-image:url(\"data:image/gif;base64,R0lGODdhHgAIALMAAAAAADU1NTk5OUJCQkpKSlZWVltbW2pqaoCAgP///wAAAAAAAAAAAAAAAAAAAAAAACH5BAkAAAoALAAAAAAeAAgAAAQw0BhFq734yjJm/p8xFEVAeGCqGERACmahgkUrvPH81cFdnjoQy8VBBTOiguSokkQAADs=\");background-repeat:no-repeat;position:absolute;right:5px;top:0;width:30px;z-index:1}.mcc-alpha-background{background-image:url('data:image/svg+xml;charset=utf-8,<svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 2 2\"><path d=\"M1 2V0h1v1H0v1z\" fill-opacity=\"0.1\"/></svg>');background-repeat:repeat;background-size:14px;bottom:0;left:0;position:absolute;right:0;top:0}.mcc-color-picker-selector{height:170px}::ng-deep mat-form-field{font-size:11px;font-weight:700}::ng-deep mat-form-field.black,::ng-deep mat-form-field.black label.mat-form-field-label{color:rgba(0,0,0,.87)!important}::ng-deep mat-form-field.black .mat-form-field-underline{background-color:rgba(0,0,0,.87)!important}::ng-deep mat-form-field.white,::ng-deep mat-form-field.white label.mat-form-field-label{color:#fff!important}::ng-deep mat-form-field.white .mat-form-field-underline{background-color:#fff!important}.mcc-color-picker-selector-preview{background-image:url('data:image/svg+xml;charset=utf-8,<svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 2 2\"><path d=\"M1 2V0h1v1H0v1z\" fill-opacity=\"0.1\"/></svg>');background-repeat:repeat;background-size:14px;border-bottom:1px solid #e1e1e1;box-sizing:content-box;height:40px;padding:15px;position:relative}.mcc-color-picker-selector-preview .mcc-color-picker-selector-preview-background{bottom:0;left:0;position:absolute;right:0;top:0}.mcc-color-picker-selector-preview form{display:inline-block;position:relative}.mcc-color-picker-selector-preview form input::-webkit-inner-spin-button,.mcc-color-picker-selector-preview form input::-webkit-outer-spin-button{-webkit-appearance:none;margin:0}.mcc-color-picker-selector-preview form input[type=number]{-moz-appearance:textfield}.mcc-color-picker-selector-preview .hex-input{margin-right:20px;width:86px}.mcc-color-picker-selector-preview .hex-input input{font-size:16px;font-weight:400}.mcc-color-picker-selector-preview .rgb-input{margin-right:4px;width:32px}.mcc-color-picker-selector-preview .rgb-input input{font-size:16px;font-weight:400}"]
                },] }
    ];
    MccColorPickerSelectorComponent.ctorParameters = function () { return [
        { type: forms.FormBuilder },
        { type: core.Renderer2 },
        { type: MccColorPickerCollectionService },
        { type: String, decorators: [{ type: core.Inject, args: [EMPTY_COLOR,] }] }
    ]; };
    MccColorPickerSelectorComponent.propDecorators = {
        _block: [{ type: core.ViewChild, args: ['block',] }],
        _bp: [{ type: core.ViewChild, args: ['blockPointer',] }],
        _bc: [{ type: core.ViewChild, args: ['blockCanvas',] }],
        _hueSelector: [{ type: core.ViewChild, args: ['hueSelector',] }],
        hueContainer: [{ type: core.ViewChild, args: ['hueContainer',] }],
        _alpha: [{ type: core.ViewChild, args: ['alpha',] }],
        _ac: [{ type: core.ViewChild, args: ['alphaContainer',] }],
        height: [{ type: core.Input }],
        selectedColor: [{ type: core.Input }],
        hideHexForms: [{ type: core.Input }],
        changeSelectedColor: [{ type: core.Output }]
    };

    /**
     * This directive change the background of the button
     */
    var MccColorPickerOptionComponent = /** @class */ (function () {
        function MccColorPickerOptionComponent(render, emptyColor, selectedColorIcon, selectedColorSvgIcon, disableSelectedIcon) {
            this.render = render;
            this.emptyColor = emptyColor;
            this.selectedColorIcon = selectedColorIcon;
            this.selectedColorSvgIcon = selectedColorSvgIcon;
            this.disableSelectedIcon = disableSelectedIcon;
            this.colorSelected = new core.EventEmitter();
            this._color = emptyColor;
        }
        Object.defineProperty(MccColorPickerOptionComponent.prototype, "color", {
            /**
             * Receive the color
             */
            get: function () {
                return this._color;
            },
            set: function (value) {
                this._color = value;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerOptionComponent.prototype, "ariaLabel", {
            get: function () {
                if (!this.color) {
                    return null;
                }
                var colorName;
                if (typeof this.color === 'string') {
                    colorName = this.color;
                }
                else {
                    colorName = this.color.text;
                }
                return colorName;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccColorPickerOptionComponent.prototype, "backgroundColor", {
            get: function () {
                if (!this.color) {
                    return null;
                }
                var color;
                if (typeof this.color === 'string') {
                    color = this.color;
                }
                else {
                    color = this.color.text;
                }
                if (parseColorString(color)) {
                    return color;
                }
            },
            enumerable: false,
            configurable: true
        });
        /**
         * returns class for "black" or "white" icon-color on a collection-item
         */
        MccColorPickerOptionComponent.prototype.getIconColorClassForColor = function (color) {
            var colorString = typeof color === 'string' ? color : color.value;
            var clr = parseColorString(colorString);
            if (clr) {
                return clr.isDark() && clr.getAlpha() > 0.3 ? 'white' : 'black';
            }
        };
        /**
         * check if a color is the current selected color
         */
        MccColorPickerOptionComponent.prototype.isColorSelected = function (color) {
            if (!color) {
                return false;
            }
            if (typeof color === 'string') {
                return this.selectedColor === color;
            }
            else {
                return this.selectedColor === color.value;
            }
        };
        return MccColorPickerOptionComponent;
    }());
    MccColorPickerOptionComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'mcc-color-picker-option',
                    template: "<button\n  #button\n  mat-mini-fab\n  role=\"option\"\n  [attr.aria-label]=\"ariaLabel\"\n  class=\"mat-elevation-z0 option-background\"\n  [disabled]=\"!color || color === emptyColor\"\n  (click)=\"colorSelected.emit(color)\"\n>\n  <span class=\"option-color\" [style.backgroundColor]=\"backgroundColor\"></span>\n\n  <ng-container *ngIf=\"!disableSelectedIcon\">\n    <mat-icon *ngIf=\"!isColorSelected(color); else selectionIcon\"></mat-icon>\n    <ng-template #selectionIcon>\n      <mat-icon\n        *ngIf=\"selectedColorSvgIcon; else selectionFontIcon\"\n        class=\"mat-svg-icon\"\n        [svgIcon]=\"selectedColorSvgIcon\"\n        [ngClass]=\"getIconColorClassForColor(color)\"\n      ></mat-icon>\n      <ng-template #selectionFontIcon>\n        <mat-icon *ngIf=\"!selectedColorSvgIcon\" class=\"mat-font-icon\" [ngClass]=\"getIconColorClassForColor(color)\">{{\n          selectedColorIcon\n        }}</mat-icon>\n      </ng-template>\n    </ng-template>\n  </ng-container>\n</button>\n",
                    preserveWhitespaces: false,
                    changeDetection: core.ChangeDetectionStrategy.OnPush,
                    styles: ["button{cursor:pointer;height:22px;margin:3px;width:22px}button mat-icon{position:relative}button mat-icon.black{color:rgba(0,0,0,.87)!important;fill:rgba(0,0,0,.87)!important}button mat-icon.white{color:#fff!important;fill:#fff!important}button.black,button.black mat-icon{color:rgba(0,0,0,.87)!important}button.white,button.white mat-icon{color:#fff!important}button .mat-font-icon{font-size:.8rem;font-weight:600;height:10px;margin:-1.3rem 0 0 -.15rem;width:10px}button .mat-svg-icon{margin:-1.3rem 0 0;width:14px}.option-background{background-color:transparent!important;background-image:url('data:image/svg+xml;charset=utf-8,<svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 2 2\"><path d=\"M1 2V0h1v1H0v1z\" fill-opacity=\"0.1\"/></svg>');background-repeat:repeat;background-size:14px;position:relative}.option-color{background-color:#d3d3d3;border-radius:12px;bottom:0;left:0;position:absolute;right:0;top:0}"]
                },] }
    ];
    MccColorPickerOptionComponent.ctorParameters = function () { return [
        { type: core.Renderer2 },
        { type: String, decorators: [{ type: core.Inject, args: [EMPTY_COLOR,] }] },
        { type: String, decorators: [{ type: core.Inject, args: [SELECTED_COLOR_ICON,] }] },
        { type: String, decorators: [{ type: core.Inject, args: [SELECTED_COLOR_SVG_ICON,] }] },
        { type: Boolean, decorators: [{ type: core.Inject, args: [DISABLE_SELECTED_COLOR_ICON,] }] }
    ]; };
    MccColorPickerOptionComponent.propDecorators = {
        color: [{ type: core.Input }],
        selectedColor: [{ type: core.Input }],
        colorSelected: [{ type: core.Output }],
        button: [{ type: core.ViewChild, args: ['button',] }]
    };

    var ɵ0 = [];
    var MccColorPickerModule = /** @class */ (function () {
        function MccColorPickerModule() {
        }
        /**
         *
         */
        MccColorPickerModule.forRoot = function (config) {
            return {
                ngModule: MccColorPickerModule,
                providers: [
                    { provide: DISABLE_SELECTED_COLOR_ICON, useValue: config.disable_selected_icon || false },
                    { provide: ENABLE_ALPHA_SELECTOR, useValue: config.enable_alpha_selector || false },
                    { provide: SELECTED_COLOR_ICON, useValue: config.selected_icon || 'done' },
                    { provide: SELECTED_COLOR_SVG_ICON, useValue: config.selected_svg_icon || null },
                    { provide: EMPTY_COLOR, useValue: 'empty_color' in config ? config.empty_color : 'none' },
                    { provide: USED_COLORS, useValue: config.used_colors || [] },
                    { provide: COLOR_STRING_FORMAT, useValue: config.color_string_format || 'hex' }
                ]
            };
        };
        return MccColorPickerModule;
    }());
    MccColorPickerModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        portal.PortalModule,
                        overlay.OverlayModule,
                        forms.ReactiveFormsModule,
                        button.MatButtonModule,
                        formField.MatFormFieldModule,
                        input.MatInputModule,
                        icon.MatIconModule
                    ],
                    declarations: [
                        MccColorPickerComponent,
                        MccConnectedColorPickerDirective,
                        MccColorPickerSelectorComponent,
                        MccColorPickerOriginDirective,
                        MccColorPickerOptionComponent,
                        MccColorPickerCollectionComponent
                    ],
                    exports: [MccColorPickerComponent, MccConnectedColorPickerDirective, MccColorPickerOriginDirective, MccColorPickerCollectionComponent],
                    providers: [
                        MccColorPickerService,
                        { provide: DISABLE_SELECTED_COLOR_ICON, useValue: false },
                        { provide: ENABLE_ALPHA_SELECTOR, useValue: false },
                        { provide: SELECTED_COLOR_ICON, useValue: 'done' },
                        { provide: SELECTED_COLOR_SVG_ICON, useValue: null },
                        { provide: EMPTY_COLOR, useValue: 'none' },
                        { provide: USED_COLORS, useValue: ɵ0 },
                        { provide: COLOR_STRING_FORMAT, useValue: 'hex' }
                    ]
                },] }
    ];

    /**
     * Generated bundle index. Do not edit.
     */

    exports.COLOR_STRING_FORMAT = COLOR_STRING_FORMAT;
    exports.DISABLE_SELECTED_COLOR_ICON = DISABLE_SELECTED_COLOR_ICON;
    exports.EMPTY_COLOR = EMPTY_COLOR;
    exports.ENABLE_ALPHA_SELECTOR = ENABLE_ALPHA_SELECTOR;
    exports.MccColorPickerCollectionComponent = MccColorPickerCollectionComponent;
    exports.MccColorPickerCollectionService = MccColorPickerCollectionService;
    exports.MccColorPickerComponent = MccColorPickerComponent;
    exports.MccColorPickerModule = MccColorPickerModule;
    exports.MccColorPickerOptionComponent = MccColorPickerOptionComponent;
    exports.MccColorPickerOriginDirective = MccColorPickerOriginDirective;
    exports.MccColorPickerSelectorComponent = MccColorPickerSelectorComponent;
    exports.MccColorPickerService = MccColorPickerService;
    exports.MccConnectedColorPickerDirective = MccConnectedColorPickerDirective;
    exports.SELECTED_COLOR_ICON = SELECTED_COLOR_ICON;
    exports.SELECTED_COLOR_SVG_ICON = SELECTED_COLOR_SVG_ICON;
    exports.USED_COLORS = USED_COLORS;
    exports.ɵ0 = ɵ0;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=material-community-components-color-picker.umd.js.map
