(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common'), require('@angular/cdk/scrolling'), require('rxjs'), require('rxjs/operators'), require('@angular/cdk/coercion')) :
    typeof define === 'function' && define.amd ? define('material-community-components/scrollspy', ['exports', '@angular/core', '@angular/common', '@angular/cdk/scrolling', 'rxjs', 'rxjs/operators', '@angular/cdk/coercion'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global['material-community-components'] = global['material-community-components'] || {}, global['material-community-components'].scrollspy = {}), global.ng.core, global.ng.common, global.ng.cdk.scrolling, global.rxjs, global.rxjs.operators, global.ng.cdk.coercion));
}(this, (function (exports, core, common, scrolling, rxjs, operators, coercion) { 'use strict';

    // scrollspy animations
    var SCROLLSPY_ANIMATION_SMOOTH = 'smooth';
    var SCROLLSPY_ANIMATION_INSTANT = 'instant';
    var SCROLLSPY_ANIMATION_AUTO = 'auto';

    var MccScrollspyService = /** @class */ (function () {
        function MccScrollspyService(window) {
            var _this = this;
            this.window = window;
            /**
             * When scroll is from click event, change this attr to true
             * So scroll event obeservable doesn't emit any update
             */
            this._fromClick = false;
            /**
             * List of scrollspy group
             */
            this.data = [];
            // listen to scroll event
            this._scrollSub = rxjs.fromEvent(window, 'scroll')
                .pipe(operators.debounceTime(50), operators.withLatestFrom(function () { return window.scrollY; }))
                .subscribe(function (position) {
                if (!_this._fromClick) {
                    _this._updateFocused(position);
                }
                _this._fromClick = false;
            });
        }
        MccScrollspyService.prototype.ngOnDestroy = function () {
            if (this._scrollSub && !this._scrollSub.closed) {
                this._scrollSub.unsubscribe();
            }
        };
        /**
         * Update information about wich element is on focus
         * @param position number
         */
        MccScrollspyService.prototype._updateFocused = function (position) {
            this.data.forEach(function (group) {
                var items = [];
                group.items.getValue().forEach(function (item, index) {
                    item.focus = false;
                    if (item.top <= position) {
                        if (items[index - 1]) {
                            items[index - 1].focus = false;
                        }
                        item.focus = true;
                    }
                    items.push(item);
                });
                group.items.next(items);
            });
        };
        /**
         * Create new group of items
         * @param name string
         * @param items MccScrollspyItemDirective[]
         * @param animation ScrollBehavior
         */
        MccScrollspyService.prototype.create = function (name, items, animation) {
            var group = this.data.find(function (group) { return group.name === name; });
            if (!group) {
                group = {
                    name: name,
                    animation: animation || SCROLLSPY_ANIMATION_SMOOTH,
                    items: new rxjs.BehaviorSubject(items || []),
                };
                this.data.push(group);
            }
            else {
                group.items.next(items || []);
            }
            return group;
        };
        /**
         * Return observable of the group
         * @param name string
         */
        MccScrollspyService.prototype.group = function (name) {
            var group = this.data.find(function (g) { return g.name === name; });
            if (!group) {
                group = this.create(name);
            }
            return group.items.asObservable();
        };
        /**
         * Scroll to one of the items
         * @param name string
         * @param id string
         */
        MccScrollspyService.prototype.scrollTo = function (name, id) {
            var _this = this;
            var group = this.data.find(function (group) { return group.name === name; });
            group.items.getValue().forEach(function (item) {
                item.focus = false;
                if (item.id === id) {
                    _this._fromClick = true;
                    _this._updateFocused(item.top);
                    window.scrollTo({ top: item.top, behavior: group.animation });
                }
            });
        };
        return MccScrollspyService;
    }());
    MccScrollspyService.decorators = [
        { type: core.Injectable }
    ];
    MccScrollspyService.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: core.Inject, args: ['Window',] }] }
    ]; };

    var MccScrollspyItemDirective = /** @class */ (function () {
        function MccScrollspyItemDirective(elementRef, renderer) {
            this.elementRef = elementRef;
            this.renderer = renderer;
        }
        Object.defineProperty(MccScrollspyItemDirective.prototype, "id", {
            get: function () {
                return this._id;
            },
            /**
             * Hold the element id, if element doesn't have id
             * the method will create one
             */
            set: function (id) {
                if (!id) {
                    id = this._createId();
                    this.renderer.setProperty(this.elementRef.nativeElement, 'id', id);
                }
                this._id = id;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccScrollspyItemDirective.prototype, "top", {
            /**
             * Element distance of the top
             */
            get: function () {
                return this.elementRef.nativeElement.offsetTop;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccScrollspyItemDirective.prototype, "focus", {
            get: function () {
                return this._focused;
            },
            /**
             * Element is focused
             */
            set: function (focused) {
                this._focused = coercion.coerceBooleanProperty(focused);
            },
            enumerable: false,
            configurable: true
        });
        MccScrollspyItemDirective.prototype.ngAfterContentInit = function () {
            if (!this.label) {
                this.label = this.elementRef.nativeElement.textContent;
            }
            this.id = this.elementRef.nativeElement.id;
        };
        /**
         * Create an ID for the element
         */
        MccScrollspyItemDirective.prototype._createId = function () {
            var tmpID = this.label.toLowerCase().replace(/[ ]+/gi, '_');
            return "mcc_scrollspy_" + tmpID;
        };
        return MccScrollspyItemDirective;
    }());
    MccScrollspyItemDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[mccScrollspyItem], [mcc-scrollspy-item]',
                    exportAs: 'mccScrollspyItem',
                },] }
    ];
    MccScrollspyItemDirective.ctorParameters = function () { return [
        { type: core.ElementRef },
        { type: core.Renderer2 }
    ]; };
    MccScrollspyItemDirective.propDecorators = {
        focus: [{ type: core.Input, args: ['focus',] }],
        label: [{ type: core.Input }]
    };
    var MccScrollspyGroupDirective = /** @class */ (function () {
        function MccScrollspyGroupDirective(mccScrollspyService) {
            this.mccScrollspyService = mccScrollspyService;
        }
        Object.defineProperty(MccScrollspyGroupDirective.prototype, "name", {
            /**
             * Name of the scrollspy group
             */
            set: function (name) {
                this._name = name;
            },
            enumerable: false,
            configurable: true
        });
        MccScrollspyGroupDirective.prototype.ngAfterContentInit = function () {
            // add the group items
            var items = this.items.map(function (item) { return item; });
            this.mccScrollspyService.create(this._name, items);
        };
        return MccScrollspyGroupDirective;
    }());
    MccScrollspyGroupDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[mccScrollspyGroup], [mcc-scrollspy-group]',
                    exportAs: 'mccScrollspyGroup',
                },] }
    ];
    MccScrollspyGroupDirective.ctorParameters = function () { return [
        { type: MccScrollspyService }
    ]; };
    MccScrollspyGroupDirective.propDecorators = {
        items: [{ type: core.ContentChildren, args: [MccScrollspyItemDirective,] }],
        name: [{ type: core.Input, args: ['mccScrollspyGroup',] }]
    };

    var ɵ0 = window;
    var MccScrollspyModule = /** @class */ (function () {
        function MccScrollspyModule() {
        }
        return MccScrollspyModule;
    }());
    MccScrollspyModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [common.CommonModule, scrolling.ScrollingModule],
                    providers: [MccScrollspyService, { provide: 'Window', useValue: ɵ0 }],
                    declarations: [MccScrollspyGroupDirective, MccScrollspyItemDirective],
                    exports: [MccScrollspyGroupDirective, MccScrollspyItemDirective],
                },] }
    ];

    /**
     * Generated bundle index. Do not edit.
     */

    exports.MccScrollspyGroupDirective = MccScrollspyGroupDirective;
    exports.MccScrollspyItemDirective = MccScrollspyItemDirective;
    exports.MccScrollspyModule = MccScrollspyModule;
    exports.MccScrollspyService = MccScrollspyService;
    exports.SCROLLSPY_ANIMATION_AUTO = SCROLLSPY_ANIMATION_AUTO;
    exports.SCROLLSPY_ANIMATION_INSTANT = SCROLLSPY_ANIMATION_INSTANT;
    exports.SCROLLSPY_ANIMATION_SMOOTH = SCROLLSPY_ANIMATION_SMOOTH;
    exports.ɵ0 = ɵ0;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=material-community-components-scrollspy.umd.js.map
