(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/common'), require('@angular/core'), require('@angular/material/button'), require('rxjs'), require('@angular/animations'), require('@angular/cdk/coercion')) :
    typeof define === 'function' && define.amd ? define('material-community-components/speed-dial', ['exports', '@angular/common', '@angular/core', '@angular/material/button', 'rxjs', '@angular/animations', '@angular/cdk/coercion'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global['material-community-components'] = global['material-community-components'] || {}, global['material-community-components']['speed-dial'] = {}), global.ng.common, global.ng.core, global.ng.material.button, global.rxjs, global.ng.animations, global.ng.cdk.coercion));
}(this, (function (exports, common, core, button, rxjs, animations, coercion) { 'use strict';

    /**
     * Default value of action button z-index
     */
    var Z_INDEX = 23;
    /**
     * SPIN animation used on speed-dial
     */
    var SPIN_ANIMATION = animations.trigger('spin', [
        animations.state('open', animations.style({
            transform: 'rotate(360deg)'
        })),
        animations.state('closed', animations.style({
            transform: 'rotate(0deg)'
        })),
        animations.transition('open => closed', [animations.animate('.3s')]),
        animations.transition('closed => open', [animations.animate('.3s')])
    ]);

    var MccSpeedDialActionsComponent = /** @class */ (function () {
        function MccSpeedDialActionsComponent(renderer) {
            this.renderer = renderer;
            this._animation = new rxjs.BehaviorSubject('scale');
        }
        Object.defineProperty(MccSpeedDialActionsComponent.prototype, "animation", {
            /**
             * Set type of animation will be executed on open/close
             * Type available are: scale | fling
             */
            set: function (animation) {
                this._animation.next(animation);
            },
            enumerable: false,
            configurable: true
        });
        /**
         * The z-index style and animation class are handle separate because
         * z-index will be set only one time, and the animation class will be set
         * every time the animation change
         */
        MccSpeedDialActionsComponent.prototype.ngAfterContentInit = function () {
            var _this = this;
            // set z-index style to each button action
            this._buttons.forEach(function (button, index) {
                _this.renderer.setStyle(button._elementRef.nativeElement, 'z-index', (Z_INDEX - index));
            });
            // set the animation class to each button action
            this._animation.subscribe(function (animation) {
                var nextAnimationClass = "speed-dial-item-animation-" + animation;
                _this._buttons.forEach(function (button) {
                    if (_this._lastAnimationClass) {
                        _this.renderer.removeClass(button._elementRef.nativeElement, _this._lastAnimationClass);
                    }
                    _this.renderer.addClass(button._elementRef.nativeElement, nextAnimationClass);
                });
                _this._lastAnimationClass = nextAnimationClass;
            });
        };
        /**
         * Responsible for change the state of the action buttons to visible
         *
         * @param direction DIRECTION
         */
        MccSpeedDialActionsComponent.prototype.show = function (direction) {
            var _this = this;
            switch (this._animation.value) {
                case 'scale': {
                    this._buttons.forEach(function (button, index) {
                        var transition = 3 + (65 * index);
                        _this.renderer.setStyle(button._elementRef.nativeElement, 'transition-delay', transition + "ms");
                        _this.renderer.setStyle(button._elementRef.nativeElement, 'opacity', '1');
                        _this.renderer.setStyle(button._elementRef.nativeElement, 'transform', 'scale(1)');
                    });
                    break;
                }
                case 'fling': {
                    var translateFn_1 = (direction == 'up' || direction == 'down') ? 'translateY' : 'translateX';
                    var sign_1 = (direction == 'down' || direction == 'right') ? '-' : '';
                    this._buttons.forEach(function (button) {
                        _this.renderer.setStyle(button._elementRef.nativeElement, 'transition-delay', '0ms');
                        _this.renderer.setStyle(button._elementRef.nativeElement, 'opacity', '1');
                        _this.renderer.setStyle(button._elementRef.nativeElement, 'transform', translateFn_1 + "(" + sign_1 + "0)");
                    });
                }
            }
        };
        /**
         * Hide all the buttons action
         *
         * @param direction DIRECTION
         */
        MccSpeedDialActionsComponent.prototype.hide = function (direction) {
            var _this = this;
            switch (this._animation.value) {
                case 'scale': {
                    this._buttons.forEach(function (button, index) {
                        var transition = 3 - (65 * index);
                        _this.renderer.setStyle(button._elementRef.nativeElement, 'transition-delay', transition + "ms");
                        _this.renderer.setStyle(button._elementRef.nativeElement, 'opacity', '0');
                        _this.renderer.setStyle(button._elementRef.nativeElement, 'transform', 'scale(0)');
                    });
                    break;
                }
                case 'fling': {
                    var translateFn_2 = (direction == 'up' || direction == 'down') ? 'translateY' : 'translateX';
                    var sign_2 = (direction == 'down' || direction == 'right') ? '-' : '';
                    this._buttons.forEach(function (button, index) {
                        var transform = (55 * (index + 1) - (index * 5));
                        _this.renderer.setStyle(button._elementRef.nativeElement, 'transition-delay', '0ms');
                        _this.renderer.setStyle(button._elementRef.nativeElement, 'opacity', '1');
                        _this.renderer.setStyle(button._elementRef.nativeElement, 'transform', translateFn_2 + "(" + sign_2 + transform + "px)");
                    });
                }
            }
        };
        return MccSpeedDialActionsComponent;
    }());
    MccSpeedDialActionsComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'mcc-speed-dial-actions',
                    template: "<ng-content select=\"button\"></ng-content>",
                    encapsulation: core.ViewEncapsulation.None,
                    styles: ["mcc-speed-dial-actions .speed-dial-item-animation-scale{transform:scale(0);transition:all .3s cubic-bezier(.55,0,.55,.2);transition-duration:.14286s}mcc-speed-dial-actions .speed-dial-item-animation-fling{display:block;opacity:1;transition:all .3s cubic-bezier(.55,0,.55,.2)}"]
                },] }
    ];
    MccSpeedDialActionsComponent.ctorParameters = function () { return [
        { type: core.Renderer2 }
    ]; };
    MccSpeedDialActionsComponent.propDecorators = {
        _buttons: [{ type: core.ContentChildren, args: [button.MatButton,] }],
        animation: [{ type: core.Input }]
    };

    var MccSpeedDialComponent = /** @class */ (function () {
        function MccSpeedDialComponent() {
            this._isOpen = false;
            this._hover = false;
            this._spin = true;
            this._direction = 'up';
            /**
             * Event emitted when open state change
             */
            this.openStateChange = new core.EventEmitter();
        }
        Object.defineProperty(MccSpeedDialComponent.prototype, "isOpen", {
            get: function () {
                return this._isOpen;
            },
            /**
             * Set initial 'open' state
             */
            set: function (open) {
                this._isOpen = coercion.coerceBooleanProperty(open);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccSpeedDialComponent.prototype, "hover", {
            /**
             * When enabled, handle open/close state on mouse hover
             */
            set: function (hover) {
                this._hover = coercion.coerceBooleanProperty(hover);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccSpeedDialComponent.prototype, "spin", {
            get: function () {
                return this._spin;
            },
            /**
             * Enable/disable spin animation when button is clicked or hovered
             */
            set: function (spin) {
                this._spin = coercion.coerceBooleanProperty(spin);
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(MccSpeedDialComponent.prototype, "direction", {
            get: function () {
                return this._direction;
            },
            /**
             * Define the direction of the actions button
             * Directions available are: up | down | left | right
             */
            set: function (direction) {
                this._direction = direction;
            },
            enumerable: false,
            configurable: true
        });
        /**
         * Call fab speed dial actions functions to change the
         * visibility of the buttons
         */
        MccSpeedDialComponent.prototype._setActionsState = function () {
            if (this._isOpen) {
                this.actions.show(this._direction);
            }
            else {
                this.actions.hide(this._direction);
            }
        };
        /**
         * Set initial state to the action buttons inside speed-dial-actions
         */
        MccSpeedDialComponent.prototype.ngAfterViewInit = function () {
            this._setActionsState();
        };
        /**
         *
         */
        MccSpeedDialComponent.prototype.ngOnChanges = function (changes) {
            if ('isOpen' in changes && changes['isOpen'].previousValue !== undefined) {
                this._setActionsState();
            }
        };
        /**
         * When mouseHover is enabled and state is closed
         * calls toggle to open the actions
         */
        MccSpeedDialComponent.prototype.hoverStart = function () {
            if (this._hover && !this._isOpen) {
                this.toggle();
            }
        };
        /**
         * When mouseHover is enabled and state is open
         * calls toggle to close the actions
         */
        MccSpeedDialComponent.prototype.hoverStop = function () {
            if (this._hover && this._isOpen) {
                this.toggle();
            }
        };
        /**
         * Change the open state
         */
        MccSpeedDialComponent.prototype.toggle = function () {
            this._isOpen = !this._isOpen;
            this._setActionsState();
            this.openStateChange.emit(this._isOpen);
        };
        return MccSpeedDialComponent;
    }());
    MccSpeedDialComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'mcc-speed-dial',
                    template: "<div class=\"mcc-speed-dial-container\" [ngClass]=\"'mcc-speed-dial-direction-' + direction\">\n  <button mat-fab [@spin]=\"spin && isOpen ? 'open' : 'closed'\" class=\"mat-elevation-z1\" (mouseenter)=\"hoverStart()\"\n    (mouseleave)=\"hoverStop()\" (click)=\"toggle()\">\n    <ng-content></ng-content>\n  </button>\n\n  <ng-content select=\"mcc-speed-dial-actions\"></ng-content>\n</div>",
                    animations: [SPIN_ANIMATION],
                    styles: [".mcc-speed-dial-container{align-items:center;display:flex;position:relative;z-index:20}.mcc-speed-dial-container button{pointer-events:auto;z-index:24}.mcc-speed-dial-container ::ng-deep mcc-speed-dial-actions{display:flex;height:auto}.mcc-speed-dial-container.mcc-speed-dial-direction-up{flex-direction:column}.mcc-speed-dial-container.mcc-speed-dial-direction-up button{order:2}.mcc-speed-dial-container.mcc-speed-dial-direction-up ::ng-deep mcc-speed-dial-actions{flex-direction:column-reverse;order:1}.mcc-speed-dial-container.mcc-speed-dial-direction-up ::ng-deep mcc-speed-dial-actions .mat-mini-fab{margin-bottom:10px}.mcc-speed-dial-container.mcc-speed-dial-direction-down{flex-direction:column}.mcc-speed-dial-container.mcc-speed-dial-direction-down button{order:1}.mcc-speed-dial-container.mcc-speed-dial-direction-down ::ng-deep mcc-speed-dial-actions{flex-direction:column;order:2}.mcc-speed-dial-container.mcc-speed-dial-direction-down ::ng-deep mcc-speed-dial-actions .mat-mini-fab{margin-top:10px}.mcc-speed-dial-container.mcc-speed-dial-direction-left{flex-direction:row}.mcc-speed-dial-container.mcc-speed-dial-direction-left button{order:2}.mcc-speed-dial-container.mcc-speed-dial-direction-left ::ng-deep mcc-speed-dial-actions{flex-direction:row-reverse;order:1}.mcc-speed-dial-container.mcc-speed-dial-direction-left ::ng-deep mcc-speed-dial-actions .mat-mini-fab{margin-right:10px}.mcc-speed-dial-container.mcc-speed-dial-direction-right{flex-direction:row}.mcc-speed-dial-container.mcc-speed-dial-direction-right button{order:1}.mcc-speed-dial-container.mcc-speed-dial-direction-right ::ng-deep mcc-speed-dial-actions{flex-direction:row;order:2}.mcc-speed-dial-container.mcc-speed-dial-direction-right ::ng-deep mcc-speed-dial-actions .mat-mini-fab{margin-left:10px}"]
                },] }
    ];
    MccSpeedDialComponent.ctorParameters = function () { return []; };
    MccSpeedDialComponent.propDecorators = {
        actions: [{ type: core.ContentChild, args: [MccSpeedDialActionsComponent,] }],
        isOpen: [{ type: core.Input, args: ['open',] }],
        hover: [{ type: core.Input, args: ['mouseHover',] }],
        spin: [{ type: core.Input }],
        direction: [{ type: core.Input }],
        openStateChange: [{ type: core.Output }]
    };

    var MccSpeedDialModule = /** @class */ (function () {
        function MccSpeedDialModule() {
        }
        return MccSpeedDialModule;
    }());
    MccSpeedDialModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [common.CommonModule, button.MatButtonModule],
                    declarations: [MccSpeedDialComponent, MccSpeedDialActionsComponent],
                    exports: [MccSpeedDialComponent, MccSpeedDialActionsComponent],
                },] }
    ];

    /**
     * Generated bundle index. Do not edit.
     */

    exports.MccSpeedDialActionsComponent = MccSpeedDialActionsComponent;
    exports.MccSpeedDialComponent = MccSpeedDialComponent;
    exports.MccSpeedDialModule = MccSpeedDialModule;
    exports.SPIN_ANIMATION = SPIN_ANIMATION;
    exports.Z_INDEX = Z_INDEX;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=material-community-components-speed-dial.umd.js.map
