import { ElementRef, EventEmitter, Renderer2 } from '@angular/core';
import { ControlValueAccessor } from '@angular/forms';
import { BehaviorSubject } from 'rxjs';
/**
 * Directive applied to an element to make it usable as an origin for an ColorPicker.
 */
export declare class MccColorPickerOriginDirective implements ControlValueAccessor {
    private elementRef;
    private renderer;
    private emptyColor;
    /**
     * Emit focus event
     */
    hasFocus: EventEmitter<void>;
    /**
     * Emit changes from the origin
     */
    change: BehaviorSubject<string>;
    /**
     * Propagate changes to angular
     */
    propagateChanges: (_: any) => {};
    /**
     * Controls if focus of the input opens the color picker dialog
     */
    openMode: 'openOnFocus' | 'default' | '';
    constructor(elementRef: ElementRef, renderer: Renderer2, emptyColor: string);
    onFocus(): void;
    onKeyup(): void;
    /**
     * This method will be called by the forms API to write to the view when
     * programmatic (model -> view) changes are requested.
     */
    writeValue(color: string): void;
    /**
     * This method will be called by the color picker
     */
    writeValueFromColorPicker(color: string): void;
    /**
     * This method will be called from origin whe key is up
     */
    writeValueFromKeyup(color: string): void;
    /**
     * This is called by the forms API on initialization so it can update the
     * form model when values propagate from the view (view -> model).
     */
    registerOnChange(fn: any): void;
    /**
     * This is called by the forms API on initialization so it can update the form model on blur
     */
    registerOnTouched(fn: any): void;
    /**
     * called by the forms API when the control status changes to or from "DISABLED"
     */
    setDisabledState(isDisabled: boolean): void;
}
