import { AfterViewInit, ElementRef, OnChanges, OnDestroy, OnInit, Renderer2, SimpleChanges } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { Observable } from 'rxjs';
import { MccColorPickerCollectionService } from './color-picker-collection.service';
export declare class MccColorPickerSelectorComponent implements AfterViewInit, OnInit, OnChanges, OnDestroy {
    private formBuilder;
    private renderer;
    colorPickerCollectionService: MccColorPickerCollectionService;
    private emptyColor;
    /**
     * ElemenRef of the main color block
     */
    _block: ElementRef;
    /**
     * ElemenRef of the pointer main color
     */
    _bp: ElementRef;
    /**
     * Canvas of the block
     */
    private _bc;
    private _blockContext;
    /**
     * ElementRef of the color base
     */
    _hueSelector: ElementRef;
    private _hueSelectorContext;
    /**
     * Container of the strip
     */
    set hueContainer(el: ElementRef);
    private _sc;
    /**
     * ElementRef of the alpha base
     */
    _alpha: ElementRef;
    private _alphaContext;
    /**
     * Container of the alpha
     */
    private _ac;
    /**
     * Change height base of the selector
     */
    set height(value: number);
    get height(): number;
    private _height;
    static ngAcceptInputType_height: number | string;
    get stripHeight(): number;
    /**
     * Receive selected color from the component
     */
    set selectedColor(value: string);
    private _selectedColor;
    /**
     * Hide the hexadecimal color forms.
     */
    set hideHexForms(value: boolean);
    get hideHexForms(): boolean;
    private _hideHexForms;
    /**
     * Emit update when a color is selected
     */
    private readonly changeSelectedColor;
    /**
     * Subject of the current selected color by the user
     */
    private _tmpSelectedColor;
    /**
     * Subscription of the tmpSelectedColor Observable
     */
    private _tmpSelectedColorSub;
    /**
     * Subscription of the hexForm values change
     */
    private _hexValuesSub;
    /**
     * Subscription of the rbgForm values change
     */
    private _rgbValuesSub;
    /**
     * Handle color of the form inputs text
     */
    textClass: string;
    /**
     * Keeps track if the mouse button is pressed
     */
    _isPressed: boolean;
    /**
     * Form of the color in hex
     */
    hexForm: FormGroup;
    /**
     * Form and keys of the fields in RGB
     */
    rgbaForm: FormGroup;
    /**
     * Last remembered coordinates in block selector
     */
    private _latestBlockCoordinates;
    /**
     * True if empty color is selected
     */
    noColor: boolean;
    /**
     * keeping track of the rectangular position on page of the current selection target
     */
    private _selectionTargetPosition;
    /**
     * keeping mousedown listeners for destruction
     */
    private _mouseDownListeners;
    /**
     * keeping track of temporary mouse listeners for destruction
     */
    private _temporaryMouseListeners;
    get selectorWidth(): number;
    private _selectorWidth;
    get stripWidth(): number;
    private _stripWidth;
    get tmpSelectedColorAsRgba$(): Observable<string>;
    constructor(formBuilder: FormBuilder, renderer: Renderer2, colorPickerCollectionService: MccColorPickerCollectionService, emptyColor: string);
    ngOnInit(): void;
    /**
     * Update forms and selectors selectedColor change and
     * the mouse button is not pressed
     */
    ngOnChanges(changes: SimpleChanges): void;
    ngOnDestroy(): void;
    ngAfterViewInit(): void;
    private _listenToMouseForBlock;
    private _listenToMouseForHue;
    private _listenToMouseForAlpha;
    private _preventTextSelection;
    private _registerMouseUpListeners;
    private _getCoordinatesForPosition;
    private _getYForPosition;
    private _drawHueSelector;
    /**
     * Generate alpha selector gradient based on the RGB color
     */
    private _drawAlphaSelector;
    /**
     * Generate color selector block based on the RGB color
     */
    private _drawBlockSelector;
    /**
     * Watch changes on forms
     */
    private _watchFormChanges;
    /**
     * Update RGBA form
     */
    private _updateRGBAForm;
    /**
     * Update hex form
     */
    private _updateHexForm;
    /**
     * Handle changes of the hue slider
     */
    private changeHue;
    /**
     * Handle changes of the alpha slider
     */
    private changeAlpha;
    /**
     * Handle changes of the selected color in the XY block
     */
    private changeColor;
    /**
     * Set all selectors positions based on a color
     */
    private setSelectorPositions;
    /**
     * Set block selector positions in view
     */
    private setXYSelector;
    /**
     * Set hue selector positions in view
     */
    private setHueSelector;
    /**
     * Set alpha selector positions in view
     */
    private setAlphaSelector;
    /**
     * Get the X and Y coordinates for the block selector relative to it's size
     */
    private getXYOffsets;
    /**
     * Get the Y coordinate for the hue selector relative to it's height
     */
    private getHueOffsets;
    /**
     * Get the Y coordinate for the alpha selector relative to it's height
     */
    private getAlphaOffset;
}
