import { AfterContentInit, ChangeDetectorRef, ElementRef, EventEmitter, OnDestroy, OnInit, QueryList } from '@angular/core';
import { Observable } from 'rxjs';
import { ColorStringFormat, MccColorPickerUsedColorPosition } from './color-picker.types';
import { MccColorPickerCollectionComponent } from './color-picker-collection.component';
import { MccColorPickerService } from './color-picker.service';
import { MccColorPickerCollectionService } from './color-picker-collection.service';
export declare class MccColorPickerComponent implements AfterContentInit, OnInit, OnDestroy {
    private elementRef;
    private changeDetectorRef;
    private colorPickerService;
    private colorPickerCollectionService;
    emptyColor: string;
    /**
     * Get all collections
     */
    _collections: QueryList<MccColorPickerCollectionComponent>;
    /**
     * Disable color-picker of showing up
     */
    set disabled(disabled: boolean);
    get disabled(): boolean;
    private _disabled;
    static ngAcceptInputType_disabled: boolean | string;
    /**
     * Change label of the collection UsedColors
     */
    get usedColorLabel(): string;
    set usedColorLabel(value: string);
    private _usedColorLabel;
    /**
     * Set initial value for used color
     */
    set usedColorStart(colors: string[]);
    /**
     * Set usedColor to be used in reverse
     */
    set reverseUsedColors(reverse: boolean);
    get reverseUsedColors(): boolean;
    private _reverseUsedColor;
    static ngAcceptInputType_reverseUsedColors: boolean | string;
    /**
     * Set position of used colors collection
     */
    set usedColorsPosition(position: MccColorPickerUsedColorPosition);
    get usedColorsPosition(): MccColorPickerUsedColorPosition;
    private _usedColorsPosition;
    /**
     * Hide the hexadecimal color forms.
     */
    set hideHexForms(value: boolean);
    get hideHexForms(): boolean;
    private _hideHexForms;
    static ngAcceptInputType_hideHexForms: boolean | string;
    /**
     * Hide empty slots from the collection UsedColors
     */
    set hideEmpty(value: boolean);
    get hideEmpty(): boolean;
    private _hideEmpty;
    static ngAcceptInputType_hideEmpty: boolean | string;
    /**
     * Hide transparent option of UsedColors
     */
    set hideTransparent(value: boolean);
    get hideTransparent(): boolean;
    private _hideTransparent;
    static ngAcceptInputType_hideTransparent: boolean | string;
    /**
     * Hide UsedColors collection
     */
    set hideUsedColors(value: boolean);
    get hideUsedColors(): boolean;
    private _hideUsedColors;
    static ngAcceptInputType_hideUsedColors: boolean | string;
    /**
     * Start with a color selected
     */
    get selectedColor(): string;
    set selectedColor(value: string);
    private _selectedColor;
    /**
     * Define if the panel will be initiated open
     */
    set isOpen(value: boolean);
    get isOpen(): boolean;
    private _isOpen;
    static ngAcceptInputType_isOpen: boolean | string;
    /**
     * Define if the panel will show in overlay or not
     */
    set overlay(value: boolean);
    get overlay(): boolean;
    private _overlay;
    static ngAcceptInputType_overlay: boolean | string;
    /**
     * Hide the action buttons (cancel/confirm)
     */
    set hideButtons(value: boolean);
    get hideButtons(): boolean;
    private _hideButtons;
    static ngAcceptInputType_hideButtons: boolean | string;
    /**
     * Define new height for the selector
     */
    set colorPickerSelectorHeight(height: number);
    get colorPickerSelectorHeight(): number;
    private _colorPickerSelectorHeight;
    static ngAcceptInputType_colorPickerSelectorHeight: number | string;
    /**
     * Hide the color picker selector
     */
    set hideColorPickerSelector(value: boolean);
    get hideColorPickerSelector(): boolean;
    private _hideColorPickerSelector;
    static ngAcceptInputType_hideColorPickerSelector: boolean | string;
    /**
     * Set the size of the used colors
     */
    usedSizeColors: number;
    static ngAcceptInputType_usedSizeColors: number | string;
    /**
     * Change btnCancel label
     */
    btnCancelLabel: string;
    /**
     * Change btnConfirm label
     */
    btnConfirmLabel: string;
    /**
     * Show alpha selector
     */
    set alpha(value: boolean);
    get alpha(): boolean;
    private _alpha;
    static ngAcceptInputType_alpha: boolean | string;
    /**
     * Choose color string format
     */
    format: ColorStringFormat;
    /**
     * Event emitted when user change the selected color (without confirm)
     */
    readonly change: EventEmitter<string>;
    /**
     * Event emitted when selected color is confirm
     */
    readonly selected: EventEmitter<string>;
    /**
     * Event emitted when is clicked outside of the component
     */
    readonly clickOut: EventEmitter<void>;
    /**
     * Event emitted when is clicked outside of the component
     */
    readonly canceled: EventEmitter<void>;
    /**
     * Return a Observable with the color the user is picking
     */
    get tmpSelectedColor$(): Observable<string>;
    private _tmpSelectedColor;
    /**
     * Observable with all the colors used by the user
     */
    get usedColors$(): Observable<string[]>;
    /**
     * Array of subscriptions from the collections
     */
    private _collectionSubs;
    constructor(elementRef: ElementRef, changeDetectorRef: ChangeDetectorRef, colorPickerService: MccColorPickerService, colorPickerCollectionService: MccColorPickerCollectionService, emptyColor: string, showAlphaSelector: boolean, colorStringFormat: ColorStringFormat);
    ngOnInit(): void;
    /**
     * Walk throw all collections and subcribe to changes
     */
    ngAfterContentInit(): void;
    /**
     * Destroy all subscriptions
     */
    ngOnDestroy(): void;
    /**
     * Update selected color and emit the change
     */
    private _updateSelectedColor;
    /**
     * Open/close color picker panel
     */
    toggle(): void;
    /**
     * Update selected color, close the panel and notify the user
     */
    backdropClick(): void;
    /**
     * Update tmpSelectedColor
     * @param color string
     */
    updateTmpSelectedColor(color: string): void;
    /**
     * Cancel the selection and close the panel
     */
    cancelSelection(): void;
    /**
     * Update selectedColor and close the panel
     */
    confirmSelectedColor(): void;
}
