import { EventEmitter, ChangeDetectorRef } from '@angular/core';
import { CdkOverlayOrigin } from '@angular/cdk/overlay';
import { Observable } from 'rxjs';
import { MccTimerPickerTimeType, MccTimerPickerFormat, MccTimerPickerHour, MccTimerPicker24Hour, MccTimerPickerMinute, MccTimerPickerPeriod, MccTimerPickerTimeValue } from './timer-picker';
export declare class MccTimerPickerComponent {
    private changeDetectorRef;
    /**
     * Receive selected _hour after confirm
     */
    private _selectedHour;
    /**
     * Receive selected _minute after confirm
     */
    private _selectedMinute;
    /**
     * Receive selected _period after confirm
     */
    private _selectedPeriod;
    /**
     * Current value (hour/minute) to create the clock
     */
    get clock$(): Observable<MccTimerPickerTimeValue[]>;
    private _clock;
    /**
     * Return hours in 24h format
     */
    get hours24(): MccTimerPicker24Hour[];
    /**
     * Type there is in focus (hour/minute)
     */
    get focus(): MccTimerPickerTimeType;
    set focus(value: MccTimerPickerTimeType);
    private _focus;
    /**
     * State of the overlay
     */
    get isOpen(): boolean;
    set isOpen(value: boolean);
    private _isOpen;
    /**
     * Return temporary selected hour (const HOURS)
     */
    get hour(): MccTimerPickerHour | MccTimerPicker24Hour;
    private _hour;
    /**
     * Checks if hour has only one digit
     */
    get needsSpacer(): boolean;
    /**
     * Return temporary selected minute (const MINUTES)
     */
    get minute(): MccTimerPickerMinute;
    private _minute;
    /**
     * Return temporary selected period (am/pm)
     */
    get period(): MccTimerPickerPeriod;
    private _period;
    /**
     * Hide Confirm and Cancel buttons
     */
    set hideButtons(value: boolean);
    get hideButtons(): boolean;
    private _hideButtons;
    static ngAcceptInputType_hideButtons: boolean | string;
    /**
     * Format of the hour to be emited on confirm
     */
    format: MccTimerPickerFormat;
    min: string;
    max: string;
    /**
     * Change btnCancel label
     */
    btnCancel: string;
    /**
     * Change btnConfirm label
     */
    btnConfirm: string;
    /**
     * Event emited when confirm button is pressed.
     * If buttons are hidden, the event is emited when value is changed
     */
    selected: EventEmitter<string>;
    /**
     * Origin reference of connected timer picker
     */
    trigger: CdkOverlayOrigin;
    /**
     * Set to true when timer picker have been connected with another component
     */
    connected: boolean;
    constructor(changeDetectorRef: ChangeDetectorRef);
    /**
     * Return timer option class to create line between the middle of the clock and
     * the option
     */
    getSelectedClass(): string;
    /**
     * Select option from the clock.
     */
    select(value: MccTimerPickerTimeValue): void;
    /**
     * Returns array containing time, hour and period fragments from time string
     */
    parseTimeInput(time: string): [number, number, string];
    /**
     * Returns true if option value is not valid
     */
    isOptionDisabled(value: MccTimerPickerTimeValue): boolean;
    /**
     * Change period of the clock
     */
    changePeriod(period: MccTimerPickerPeriod): void;
    /**
     * Update selected color, close the panel and notify the user
     */
    backdropClick(): void;
    /**
     * Change values to last confirm select time
     */
    cancelSelection(): void;
    /**
     * Set new values of time and emit new event with the formatted timer
     */
    confirmSelectedTime(): void;
}
